@@ -1,9 +1,9 @@
 # Natural Language Toolkit: WordNet Browser Application
 #
-# Copyright (C) 2001-2019 NLTK Project
+# Copyright (C) 2001-2022 NLTK Project
 # Author: Jussi Salmela <jtsalmela@users.sourceforge.net>
 #         Paul Bone <pbone@students.csse.unimelb.edu.au>
-# URL: <http://nltk.org/>
+# URL: <https://www.nltk.org/>
 # For license information, see LICENSE.TXT
 
 """
@@ -38,41 +38,33 @@ Options::
 
     -s or --server-mode
         Do not start a web browser, and do not allow a user to
-        shotdown the server through the web interface.
+        shutdown the server through the web interface.
 """
 # TODO: throughout this package variable names and docstrings need
 # modifying to be compliant with NLTK's coding standards.  Tests also
 # need to be develop to ensure this continues to work in the face of
 # changes to other NLTK packages.
-from __future__ import print_function
 
-# Allow this program to run inside the NLTK source tree.
-from sys import path
-
-import os
-import sys
-from sys import argv
-from collections import defaultdict
-import webbrowser
+import base64
+import copy
 import datetime
+import getopt
+import os
+import pickle
 import re
+import sys
 import threading
 import time
-import getopt
-import base64
-import pickle
-import copy
+import webbrowser
+from collections import defaultdict
+from http.server import BaseHTTPRequestHandler, HTTPServer
 
-from six.moves.urllib.parse import unquote_plus
+# Allow this program to run inside the NLTK source tree.
+from sys import argv, path
+from urllib.parse import unquote_plus
 
-from nltk import compat
 from nltk.corpus import wordnet as wn
-from nltk.corpus.reader.wordnet import Synset, Lemma
-
-if compat.PY3:
-    from http.server import HTTPServer, BaseHTTPRequestHandler
-else:
-    from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler
+from nltk.corpus.reader.wordnet import Lemma, Synset
 
 # now included in local file
 # from util import html_header, html_trailer, \
@@ -85,7 +77,7 @@ firstClient = True
 # gets set by demo().
 server_mode = None
 
-# If set this is a file object for writting log messages.
+# If set this is a file object for writing log messages.
 logfile = None
 
 
@@ -96,40 +88,40 @@ class MyServerHandler(BaseHTTPRequestHandler):
     def do_GET(self):
         global firstClient
         sp = self.path[1:]
-        if unquote_plus(sp) == 'SHUTDOWN THE SERVER':
+        if unquote_plus(sp) == "SHUTDOWN THE SERVER":
             if server_mode:
                 page = "Server must be killed with SIGTERM."
                 type = "text/plain"
             else:
-                print('Server shutting down!')
+                print("Server shutting down!")
                 os._exit(0)
 
-        elif sp == '':  # First request.
-            type = 'text/html'
+        elif sp == "":  # First request.
+            type = "text/html"
             if not server_mode and firstClient:
                 firstClient = False
                 page = get_static_index_page(True)
             else:
                 page = get_static_index_page(False)
-            word = 'green'
+            word = "green"
 
-        elif sp.endswith('.html'):  # Trying to fetch a HTML file TODO:
-            type = 'text/html'
+        elif sp.endswith(".html"):  # Trying to fetch a HTML file TODO:
+            type = "text/html"
             usp = unquote_plus(sp)
-            if usp == 'NLTK Wordnet Browser Database Info.html':
-                word = '* Database Info *'
+            if usp == "NLTK Wordnet Browser Database Info.html":
+                word = "* Database Info *"
                 if os.path.isfile(usp):
-                    with open(usp, 'r') as infile:
+                    with open(usp) as infile:
                         page = infile.read()
                 else:
                     page = (
-                        (html_header % word) + '<p>The database info file:'
-                        '<p><b>'
+                        (html_header % word) + "<p>The database info file:"
+                        "<p><b>"
                         + usp
-                        + '</b>'
-                        + '<p>was not found. Run this:'
-                        + '<p><b>python dbinfo_html.py</b>'
-                        + '<p>to produce it.'
+                        + "</b>"
+                        + "<p>was not found. Run this:"
+                        + "<p><b>python dbinfo_html.py</b>"
+                        + "<p>to produce it."
                         + html_trailer
                     )
             else:
@@ -138,7 +130,7 @@ class MyServerHandler(BaseHTTPRequestHandler):
                 page = get_static_page_by_path(usp)
         elif sp.startswith("search"):
             # This doesn't seem to work with MWEs.
-            type = 'text/html'
+            type = "text/html"
             parts = (sp.split("?")[1]).split("&")
             word = [
                 p.split("=")[1].replace("+", " ")
@@ -148,25 +140,25 @@ class MyServerHandler(BaseHTTPRequestHandler):
             page, word = page_from_word(word)
         elif sp.startswith("lookup_"):
             # TODO add a variation of this that takes a non ecoded word or MWE.
-            type = 'text/html'
+            type = "text/html"
             sp = sp[len("lookup_") :]
             page, word = page_from_href(sp)
         elif sp == "start_page":
             # if this is the first request we should display help
             # information, and possibly set a default word.
-            type = 'text/html'
+            type = "text/html"
             page, word = page_from_word("wordnet")
         else:
-            type = 'text/plain'
+            type = "text/plain"
             page = "Could not parse request: '%s'" % sp
 
         # Send result.
         self.send_head(type)
-        self.wfile.write(page.encode('utf8'))
+        self.wfile.write(page.encode("utf8"))
 
     def send_head(self, type=None):
         self.send_response(200)
-        self.send_header('Content-type', type)
+        self.send_header("Content-type", type)
         self.end_headers()
 
     def log_message(self, format, *args):
@@ -184,7 +176,7 @@ def get_unique_counter_from_url(sp):
     Extract the unique counter from the URL if it has one.  Otherwise return
     null.
     """
-    pos = sp.rfind('%23')
+    pos = sp.rfind("%23")
     if pos != -1:
         return int(sp[(pos + 3) :])
     else:
@@ -227,14 +219,14 @@ def wnb(port=8000, runBrowser=True, logfilename=None):
     if logfilename:
         try:
             logfile = open(logfilename, "a", 1)  # 1 means 'line buffering'
-        except IOError as e:
+        except OSError as e:
             sys.stderr.write("Couldn't open %s for writing: %s", logfilename, e)
             sys.exit(1)
     else:
         logfile = None
 
     # Compute URL and start web browser
-    url = 'http://localhost:' + str(port)
+    url = "http://localhost:" + str(port)
 
     server_ready = None
     browser_thread = None
@@ -244,9 +236,9 @@ def wnb(port=8000, runBrowser=True, logfilename=None):
         browser_thread = startBrowser(url, server_ready)
 
     # Start the server.
-    server = HTTPServer(('', port), MyServerHandler)
+    server = HTTPServer(("", port), MyServerHandler)
     if logfile:
-        logfile.write('NLTK Wordnet browser server running serving: %s\n' % url)
+        logfile.write("NLTK Wordnet browser server running serving: %s\n" % url)
     if runBrowser:
         server_ready.set()
 
@@ -294,10 +286,10 @@ This provides a backend to both wxbrowse and browserver.py.
 # WordNet corpus is installed.
 def _pos_tuples():
     return [
-        (wn.NOUN, 'N', 'noun'),
-        (wn.VERB, 'V', 'verb'),
-        (wn.ADJ, 'J', 'adj'),
-        (wn.ADV, 'R', 'adv'),
+        (wn.NOUN, "N", "noun"),
+        (wn.VERB, "V", "verb"),
+        (wn.ADJ, "J", "adj"),
+        (wn.ADV, "R", "adv"),
     ]
 
 
@@ -307,8 +299,8 @@ def _pos_match(pos_tuple):
     tuple given to it.  It attempts to match it against the first
     non-null component of the given pos tuple.
     """
-    if pos_tuple[0] == 's':
-        pos_tuple = ('a', pos_tuple[1], pos_tuple[2])
+    if pos_tuple[0] == "s":
+        pos_tuple = ("a", pos_tuple[1], pos_tuple[2])
     for n, x in enumerate(pos_tuple):
         if x is not None:
             break
@@ -373,24 +365,24 @@ def get_relations_data(word, synset):
     """
     if synset.pos() == wn.NOUN:
         return (
-            (HYPONYM, 'Hyponyms', synset.hyponyms()),
-            (INSTANCE_HYPONYM, 'Instance hyponyms', synset.instance_hyponyms()),
-            (HYPERNYM, 'Direct hypernyms', synset.hypernyms()),
+            (HYPONYM, "Hyponyms", synset.hyponyms()),
+            (INSTANCE_HYPONYM, "Instance hyponyms", synset.instance_hyponyms()),
+            (HYPERNYM, "Direct hypernyms", synset.hypernyms()),
             (
                 INDIRECT_HYPERNYMS,
-                'Indirect hypernyms',
+                "Indirect hypernyms",
                 rebuild_tree(synset.tree(lambda x: x.hypernyms()))[1],
             ),
             #  hypernyms', 'Sister terms',
-            (INSTANCE_HYPERNYM, 'Instance hypernyms', synset.instance_hypernyms()),
+            (INSTANCE_HYPERNYM, "Instance hypernyms", synset.instance_hypernyms()),
             #            (CLASS_REGIONAL, ['domain term region'], ),
-            (PART_HOLONYM, 'Part holonyms', synset.part_holonyms()),
-            (PART_MERONYM, 'Part meronyms', synset.part_meronyms()),
-            (SUBSTANCE_HOLONYM, 'Substance holonyms', synset.substance_holonyms()),
-            (SUBSTANCE_MERONYM, 'Substance meronyms', synset.substance_meronyms()),
-            (MEMBER_HOLONYM, 'Member holonyms', synset.member_holonyms()),
-            (MEMBER_MERONYM, 'Member meronyms', synset.member_meronyms()),
-            (ATTRIBUTE, 'Attributes', synset.attributes()),
+            (PART_HOLONYM, "Part holonyms", synset.part_holonyms()),
+            (PART_MERONYM, "Part meronyms", synset.part_meronyms()),
+            (SUBSTANCE_HOLONYM, "Substance holonyms", synset.substance_holonyms()),
+            (SUBSTANCE_MERONYM, "Substance meronyms", synset.substance_meronyms()),
+            (MEMBER_HOLONYM, "Member holonyms", synset.member_holonyms()),
+            (MEMBER_MERONYM, "Member meronyms", synset.member_meronyms()),
+            (ATTRIBUTE, "Attributes", synset.attributes()),
             (ANTONYM, "Antonyms", lemma_property(word, synset, lambda l: l.antonyms())),
             (
                 DERIVATIONALLY_RELATED_FORM,
@@ -402,18 +394,18 @@ def get_relations_data(word, synset):
         )
     elif synset.pos() == wn.VERB:
         return (
-            (ANTONYM, 'Antonym', lemma_property(word, synset, lambda l: l.antonyms())),
-            (HYPONYM, 'Hyponym', synset.hyponyms()),
-            (HYPERNYM, 'Direct hypernyms', synset.hypernyms()),
+            (ANTONYM, "Antonym", lemma_property(word, synset, lambda l: l.antonyms())),
+            (HYPONYM, "Hyponym", synset.hyponyms()),
+            (HYPERNYM, "Direct hypernyms", synset.hypernyms()),
             (
                 INDIRECT_HYPERNYMS,
-                'Indirect hypernyms',
+                "Indirect hypernyms",
                 rebuild_tree(synset.tree(lambda x: x.hypernyms()))[1],
             ),
-            (ENTAILMENT, 'Entailments', synset.entailments()),
-            (CAUSE, 'Causes', synset.causes()),
-            (ALSO_SEE, 'Also see', synset.also_sees()),
-            (VERB_GROUP, 'Verb Groups', synset.verb_groups()),
+            (ENTAILMENT, "Entailments", synset.entailments()),
+            (CAUSE, "Causes", synset.causes()),
+            (ALSO_SEE, "Also see", synset.also_sees()),
+            (VERB_GROUP, "Verb Groups", synset.verb_groups()),
             (
                 DERIVATIONALLY_RELATED_FORM,
                 "Derivationally related form",
@@ -424,29 +416,29 @@ def get_relations_data(word, synset):
         )
     elif synset.pos() == wn.ADJ or synset.pos == wn.ADJ_SAT:
         return (
-            (ANTONYM, 'Antonym', lemma_property(word, synset, lambda l: l.antonyms())),
-            (SIMILAR, 'Similar to', synset.similar_tos()),
+            (ANTONYM, "Antonym", lemma_property(word, synset, lambda l: l.antonyms())),
+            (SIMILAR, "Similar to", synset.similar_tos()),
             # Participle of verb - not supported by corpus
             (
                 PERTAINYM,
-                'Pertainyms',
+                "Pertainyms",
                 lemma_property(word, synset, lambda l: l.pertainyms()),
             ),
-            (ATTRIBUTE, 'Attributes', synset.attributes()),
-            (ALSO_SEE, 'Also see', synset.also_sees()),
+            (ATTRIBUTE, "Attributes", synset.attributes()),
+            (ALSO_SEE, "Also see", synset.also_sees()),
         )
     elif synset.pos() == wn.ADV:
         # This is weird. adverbs such as 'quick' and 'fast' don't seem
         # to have antonyms returned by the corpus.a
         return (
-            (ANTONYM, 'Antonym', lemma_property(word, synset, lambda l: l.antonyms())),
+            (ANTONYM, "Antonym", lemma_property(word, synset, lambda l: l.antonyms())),
         )
         # Derived from adjective - not supported by corpus
     else:
         raise TypeError("Unhandles synset POS type: " + str(synset.pos()))
 
 
-html_header = '''
+html_header = """
 <!DOCTYPE html PUBLIC '-//W3C//DTD HTML 4.01//EN'
 'http://www.w3.org/TR/html4/strict.dtd'>
 <html>
@@ -457,13 +449,13 @@ html_header = '''
 'text/html; charset=us-ascii'>
 <title>NLTK Wordnet Browser display of: %s</title></head>
 <body bgcolor='#F5F5F5' text='#000000'>
-'''
-html_trailer = '''
+"""
+html_trailer = """
 </body>
 </html>
-'''
+"""
 
-explanation = '''
+explanation = """
 <h3>Search Help</h3>
 <ul><li>The display below the line is an example of the output the browser
 shows you when you enter a search word. The search word was <b>green</b>.</li>
@@ -482,33 +474,33 @@ synsets.</li>
 <b>Enter/Return</b> key or click the <b>Search</b> button.</li>
 </ul>
 <hr width='100%'>
-'''
+"""
 
 # HTML oriented functions
 
 
 def _bold(txt):
-    return '<b>%s</b>' % txt
+    return "<b>%s</b>" % txt
 
 
 def _center(txt):
-    return '<center>%s</center>' % txt
+    return "<center>%s</center>" % txt
 
 
 def _hlev(n, txt):
-    return '<h%d>%s</h%d>' % (n, txt, n)
+    return "<h%d>%s</h%d>" % (n, txt, n)
 
 
 def _italic(txt):
-    return '<i>%s</i>' % txt
+    return "<i>%s</i>" % txt
 
 
 def _li(txt):
-    return '<li>%s</li>' % txt
+    return "<li>%s</li>" % txt
 
 
 def pg(word, body):
-    '''
+    """
     Return a HTML page of NLTK Browser format constructed from the
     word and body
 
@@ -518,34 +510,34 @@ def pg(word, body):
     :type body: str
     :return: a HTML page for the word-body combination
     :rtype: str
-    '''
+    """
     return (html_header % word) + body + html_trailer
 
 
 def _ul(txt):
-    return '<ul>' + txt + '</ul>'
+    return "<ul>" + txt + "</ul>"
 
 
 def _abbc(txt):
     """
     abbc = asterisks, breaks, bold, center
     """
-    return _center(_bold('<br>' * 10 + '*' * 10 + ' ' + txt + ' ' + '*' * 10))
+    return _center(_bold("<br>" * 10 + "*" * 10 + " " + txt + " " + "*" * 10))
 
 
-full_hyponym_cont_text = _ul(_li(_italic('(has full hyponym continuation)'))) + '\n'
+full_hyponym_cont_text = _ul(_li(_italic("(has full hyponym continuation)"))) + "\n"
 
 
 def _get_synset(synset_key):
     """
     The synset key is the unique name of the synset, this can be
-    retrived via synset.name()
+    retrieved via synset.name()
     """
     return wn.synset(synset_key)
 
 
 def _collect_one_synset(word, synset, synset_relations):
-    '''
+    """
     Returns the HTML string for one synset or word
 
     :param word: the current word
@@ -557,11 +549,11 @@ def _collect_one_synset(word, synset, synset_relations):
     :type synset_relations: dict(synset_key, set(relation_id))
     :return: The HTML string built for this synset
     :rtype: str
-    '''
+    """
     if isinstance(synset, tuple):  # It's a word
         raise NotImplementedError("word not supported by _collect_one_synset")
 
-    typ = 'S'
+    typ = "S"
     pos_tuple = _pos_match((synset.pos(), None, None))
     assert pos_tuple is not None, "pos_tuple is null: synset.pos(): %s" % synset.pos()
     descr = pos_tuple[2]
@@ -570,23 +562,23 @@ def _collect_one_synset(word, synset, synset_relations):
     synset_label = typ + ";"
     if synset.name() in synset_relations:
         synset_label = _bold(synset_label)
-    s = '<li>%s (%s) ' % (make_lookup_link(ref, synset_label), descr)
+    s = f"<li>{make_lookup_link(ref, synset_label)} ({descr}) "
 
     def format_lemma(w):
-        w = w.replace('_', ' ')
+        w = w.replace("_", " ")
         if w.lower() == word:
             return _bold(w)
         else:
             ref = Reference(w)
             return make_lookup_link(ref, w)
 
-    s += ', '.join(format_lemma(l.name()) for l in synset.lemmas())
+    s += ", ".join(format_lemma(l.name()) for l in synset.lemmas())
 
-    gl = " (%s) <i>%s</i> " % (
+    gl = " ({}) <i>{}</i> ".format(
         synset.definition(),
-        "; ".join("\"%s\"" % e for e in synset.examples()),
+        "; ".join('"%s"' % e for e in synset.examples()),
     )
-    return s + gl + _synset_relations(word, synset, synset_relations) + '</li>\n'
+    return s + gl + _synset_relations(word, synset, synset_relations) + "</li>\n"
 
 
 def _collect_all_synsets(word, pos, synset_relations=dict()):
@@ -594,16 +586,14 @@ def _collect_all_synsets(word, pos, synset_relations=dict()):
     Return a HTML unordered list of synsets for the given word and
     part of speech.
     """
-    return '<ul>%s\n</ul>\n' % ''.join(
-        (
-            _collect_one_synset(word, synset, synset_relations)
-            for synset in wn.synsets(word, pos)
-        )
+    return "<ul>%s\n</ul>\n" % "".join(
+        _collect_one_synset(word, synset, synset_relations)
+        for synset in wn.synsets(word, pos)
     )
 
 
 def _synset_relations(word, synset, synset_relations):
-    '''
+    """
     Builds the HTML string for the relations of a synset
 
     :param word: The current word
@@ -614,7 +604,7 @@ def _synset_relations(word, synset, synset_relations):
     :type synset_relations: dict(synset_key, set(relation_type))
     :return: The HTML for a synset's relations
     :rtype: str
-    '''
+    """
 
     if not synset.name() in synset_relations:
         return ""
@@ -628,9 +618,9 @@ def _synset_relations(word, synset, synset_relations):
         elif isinstance(r, tuple):
             # It's probably a tuple containing a Synset and a list of
             # similar tuples.  This forms a tree of synsets.
-            return "%s\n<ul>%s</ul>\n" % (
+            return "{}\n<ul>{}</ul>\n".format(
                 relation_html(r[0]),
-                ''.join('<li>%s</li>\n' % relation_html(sr) for sr in r[1]),
+                "".join("<li>%s</li>\n" % relation_html(sr) for sr in r[1]),
             )
         else:
             raise TypeError(
@@ -639,34 +629,32 @@ def _synset_relations(word, synset, synset_relations):
             )
 
     def make_synset_html(db_name, disp_name, rels):
-        synset_html = '<i>%s</i>\n' % make_lookup_link(
+        synset_html = "<i>%s</i>\n" % make_lookup_link(
             copy.deepcopy(ref).toggle_synset_relation(synset, db_name).encode(),
             disp_name,
         )
 
         if db_name in ref.synset_relations[synset.name()]:
-            synset_html += '<ul>%s</ul>\n' % ''.join(
+            synset_html += "<ul>%s</ul>\n" % "".join(
                 "<li>%s</li>\n" % relation_html(r) for r in rels
             )
 
         return synset_html
 
     html = (
-        '<ul>'
-        + '\n'.join(
-            (
-                "<li>%s</li>" % make_synset_html(*rel_data)
-                for rel_data in get_relations_data(word, synset)
-                if rel_data[2] != []
-            )
+        "<ul>"
+        + "\n".join(
+            "<li>%s</li>" % make_synset_html(*rel_data)
+            for rel_data in get_relations_data(word, synset)
+            if rel_data[2] != []
         )
-        + '</ul>'
+        + "</ul>"
     )
 
     return html
 
 
-class Reference(object):
+class Reference:
     """
     A reference to a page that may be generated by page_word
     """
@@ -732,7 +720,7 @@ class Reference(object):
 
 
 def make_lookup_link(ref, label):
-    return '<a href="lookup_%s">%s</a>' % (ref.encode(), label)
+    return f'<a href="lookup_{ref.encode()}">{label}</a>'
 
 
 def page_from_word(word):
@@ -750,7 +738,7 @@ def page_from_word(word):
 
 
 def page_from_href(href):
-    '''
+    """
     Returns a tuple of the HTML page built and the new current word
 
     :param href: The hypertext reference to be solved
@@ -759,12 +747,12 @@ def page_from_href(href):
              to be sent to the browser and
              word is the new current word
     :rtype: A tuple (str,str)
-    '''
+    """
     return page_from_reference(Reference.decode(href))
 
 
 def page_from_reference(href):
-    '''
+    """
     Returns a tuple of the HTML page built and the new current word
 
     :param href: The hypertext reference to be solved
@@ -773,11 +761,11 @@ def page_from_reference(href):
              to be sent to the browser and
              word is the new current word
     :rtype: A tuple (str,str)
-    '''
+    """
     word = href.word
     pos_forms = defaultdict(list)
-    words = word.split(',')
-    words = [w for w in [w.strip().lower().replace(' ', '_') for w in words] if w != ""]
+    words = word.split(",")
+    words = [w for w in [w.strip().lower().replace(" ", "_") for w in words] if w != ""]
     if len(words) == 0:
         # No words were found.
         return "", "Please specify a word to search for."
@@ -789,10 +777,10 @@ def page_from_reference(href):
             form = wn.morphy(w, pos)
             if form and form not in pos_forms[pos]:
                 pos_forms[pos].append(form)
-    body = ''
+    body = ""
     for pos, pos_str, name in _pos_tuples():
         if pos in pos_forms:
-            body += _hlev(3, name) + '\n'
+            body += _hlev(3, name) + "\n"
             for w in pos_forms[pos]:
                 # Not all words of exc files are in the database, skip
                 # to the next word if a KeyError is raised.
@@ -801,7 +789,7 @@ def page_from_reference(href):
                 except KeyError:
                     pass
     if not body:
-        body = "The word or words '%s' where not found in the dictonary." % word
+        body = "The word or words '%s' where not found in the dictionary." % word
     return body, word
 
 
@@ -840,9 +828,9 @@ def get_static_web_help_page():
 <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
 <html>
      <!-- Natural Language Toolkit: Wordnet Interface: Graphical Wordnet Browser
-            Copyright (C) 2001-2019 NLTK Project
+            Copyright (C) 2001-2022 NLTK Project
             Author: Jussi Salmela <jtsalmela@users.sourceforge.net>
-            URL: <http://nltk.org/>
+            URL: <https://www.nltk.org/>
             For license information, see LICENSE.TXT -->
      <head>
           <meta http-equiv='Content-Type' content='text/html; charset=us-ascii'>
@@ -852,8 +840,8 @@ def get_static_web_help_page():
 <h2>NLTK Wordnet Browser Help</h2>
 <p>The NLTK Wordnet Browser is a tool to use in browsing the Wordnet database. It tries to behave like the Wordnet project's web browser but the difference is that the NLTK Wordnet Browser uses a local Wordnet database.
 <p><b>You are using the Javascript client part of the NLTK Wordnet BrowseServer.</b> We assume your browser is in tab sheets enabled mode.</p>
-<p>For background information on Wordnet, see the Wordnet project home page: <a href="http://wordnet.princeton.edu/"><b> http://wordnet.princeton.edu/</b></a>. For more information on the NLTK project, see the project home:
-<a href="http://nltk.sourceforge.net/"><b>http://nltk.sourceforge.net/</b></a>. To get an idea of what the Wordnet version used by this browser includes choose <b>Show Database Info</b> from the <b>View</b> submenu.</p>
+<p>For background information on Wordnet, see the Wordnet project home page: <a href="https://wordnet.princeton.edu/"><b> https://wordnet.princeton.edu/</b></a>. For more information on the NLTK project, see the project home:
+<a href="https://www.nltk.org/"><b>https://www.nltk.org/</b></a>. To get an idea of what the Wordnet version used by this browser includes choose <b>Show Database Info</b> from the <b>View</b> submenu.</p>
 <h3>Word search</h3>
 <p>The word to be searched is typed into the <b>New Word</b> field and the search started with Enter or by clicking the <b>Search</b> button. There is no uppercase/lowercase distinction: the search word is transformed to lowercase before the search.</p>
 <p>In addition, the word does not have to be in base form. The browser tries to find the possible base form(s) by making certain morphological substitutions. Typing <b>fLIeS</b> as an obscure example gives one <a href="MfLIeS">this</a>. Click the previous link to see what this kind of search looks like and then come back to this page by using the <b>Alt+LeftArrow</b> key combination.</p>
@@ -910,9 +898,9 @@ def get_static_index_page(with_shutdown):
 <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Frameset//EN"  "http://www.w3.org/TR/html4/frameset.dtd">
 <HTML>
      <!-- Natural Language Toolkit: Wordnet Interface: Graphical Wordnet Browser
-            Copyright (C) 2001-2019 NLTK Project
+            Copyright (C) 2001-2022 NLTK Project
             Author: Jussi Salmela <jtsalmela@users.sourceforge.net>
-            URL: <http://nltk.org/>
+            URL: <https://www.nltk.org/>
             For license information, see LICENSE.TXT -->
      <HEAD>
          <TITLE>NLTK Wordnet Browser</TITLE>
@@ -943,9 +931,9 @@ def get_static_upper_page(with_shutdown):
 <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
 <html>
     <!-- Natural Language Toolkit: Wordnet Interface: Graphical Wordnet Browser
-        Copyright (C) 2001-2019 NLTK Project
+        Copyright (C) 2001-2022 NLTK Project
         Author: Jussi Salmela <jtsalmela@users.sourceforge.net>
-        URL: <http://nltk.org/>
+        URL: <https://www.nltk.org/>
         For license information, see LICENSE.TXT -->
     <head>
                 <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
@@ -964,7 +952,7 @@ def get_static_upper_page(with_shutdown):
 </html>
 """
     if with_shutdown:
-        shutdown_link = "<a href=\"SHUTDOWN THE SERVER\">Shutdown</a>"
+        shutdown_link = '<a href="SHUTDOWN THE SERVER">Shutdown</a>'
     else:
         shutdown_link = ""
 
@@ -1003,7 +991,7 @@ def app():
         wnb(port, not server_mode, logfilename)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     app()
 
-__all__ = ['app']
+__all__ = ["app"]
