@@ -1,22 +1,22 @@
 # Natural Language Toolkit: Regular Expression Chunkers
 #
-# Copyright (C) 2001-2020 NLTK Project
+# Copyright (C) 2001-2022 NLTK Project
 # Author: Edward Loper <edloper@gmail.com>
 #         Steven Bird <stevenbird1@gmail.com> (minor additions)
-# URL: <http://nltk.org/>
+# URL: <https://www.nltk.org/>
 # For license information, see LICENSE.TXT
 
 import re
 
-from nltk.tree import Tree
 from nltk.chunk.api import ChunkParserI
+from nltk.tree import Tree
 
-##//////////////////////////////////////////////////////
-##  ChunkString
-##//////////////////////////////////////////////////////
+# //////////////////////////////////////////////////////
+# ChunkString
+# //////////////////////////////////////////////////////
 
 
-class ChunkString(object):
+class ChunkString:
     """
     A string-based encoding of a particular chunking of a text.
     Internally, the ``ChunkString`` class uses a single string to
@@ -50,21 +50,21 @@ class ChunkString(object):
 
     :cvar IN_CHUNK_PATTERN: A zero-width regexp pattern string that
         will only match positions that are in chunks.
-    :cvar IN_CHINK_PATTERN: A zero-width regexp pattern string that
-        will only match positions that are in chinks.
+    :cvar IN_STRIP_PATTERN: A zero-width regexp pattern string that
+        will only match positions that are in strips.
     """
 
     CHUNK_TAG_CHAR = r"[^\{\}<>]"
     CHUNK_TAG = r"(<%s+?>)" % CHUNK_TAG_CHAR
 
     IN_CHUNK_PATTERN = r"(?=[^\{]*\})"
-    IN_CHINK_PATTERN = r"(?=[^\}]*(\{|$))"
+    IN_STRIP_PATTERN = r"(?=[^\}]*(\{|$))"
 
     # These are used by _verify
     _CHUNK = r"(\{%s+?\})+?" % CHUNK_TAG
-    _CHINK = r"(%s+?)+?" % CHUNK_TAG
+    _STRIP = r"(%s+?)+?" % CHUNK_TAG
     _VALID = re.compile(r"^(\{?%s\}?)*?$" % CHUNK_TAG)
-    _BRACKETS = re.compile("[^\{\}]+")
+    _BRACKETS = re.compile(r"[^\{\}]+")
     _BALANCED_BRACKETS = re.compile(r"(\{\})*$")
 
     def __init__(self, chunk_struct, debug_level=1):
@@ -78,12 +78,14 @@ class ChunkString(object):
         :param debug_level: The level of debugging which should be
             applied to transformations on the ``ChunkString``.  The
             valid levels are:
+
                 - 0: no checks
                 - 1: full check on to_chunkstruct
                 - 2: full check on to_chunkstruct and cursory check after
-                   each transformation.
+                  each transformation.
                 - 3: full check on to_chunkstruct and full check after
-                   each transformation.
+                  each transformation.
+
             We recommend you use at least level 1.  You should
             probably use level 3 if you use any non-standard
             subclasses of ``RegexpChunkRule``.
@@ -209,7 +211,7 @@ class ChunkString(object):
         # The substitution might have generated "empty chunks"
         # (substrings of the form "{}").  Remove them, so they don't
         # interfere with other transformations.
-        s = re.sub("\{\}", "", s)
+        s = re.sub(r"\{\}", "", s)
 
         # Make sure that the transformation was legal.
         if self._debug > 1:
@@ -236,7 +238,7 @@ class ChunkString(object):
         tags will line up with the representation of other
         ``ChunkStrings`` for the same text, regardless of the chunking.
 
-       :rtype: str
+        :rtype: str
         """
         # Add spaces to make everything line up.
         str = re.sub(r">(?!\})", r"> ", self._str)
@@ -246,12 +248,12 @@ class ChunkString(object):
         return str
 
 
-##//////////////////////////////////////////////////////
-##  Chunking Rules
-##//////////////////////////////////////////////////////
+# //////////////////////////////////////////////////////
+# Chunking Rules
+# //////////////////////////////////////////////////////
 
 
-class RegexpChunkRule(object):
+class RegexpChunkRule:
     """
     A rule specifying how to modify the chunking in a ``ChunkString``,
     using a transformational regular expression.  The
@@ -352,7 +354,7 @@ class RegexpChunkRule(object):
         Currently, the following formats are supported::
 
           {regexp}         # chunk rule
-          }regexp{         # chink rule
+          }regexp{         # strip rule
           regexp}{regexp   # split rule
           regexp{}regexp   # merge rule
 
@@ -369,14 +371,14 @@ class RegexpChunkRule(object):
         rule = m.group("rule").strip()
         comment = (m.group("comment") or "")[1:].strip()
 
-        # Pattern bodies: chunk, chink, split, merge
+        # Pattern bodies: chunk, strip, split, merge
         try:
             if not rule:
                 raise ValueError("Empty chunk pattern")
             if rule[0] == "{" and rule[-1] == "}":
                 return ChunkRule(rule[1:-1], comment)
             elif rule[0] == "}" and rule[-1] == "{":
-                return ChinkRule(rule[1:-1], comment)
+                return StripRule(rule[1:-1], comment)
             elif "}{" in rule:
                 left, right = rule.split("}{")
                 return SplitRule(left, right, comment)
@@ -388,8 +390,8 @@ class RegexpChunkRule(object):
                 return ChunkRuleWithContext(left, chunk, right, comment)
             else:
                 raise ValueError("Illegal chunk pattern: %s" % rule)
-        except (ValueError, re.error):
-            raise ValueError("Illegal chunk pattern: %s" % rule)
+        except (ValueError, re.error) as e:
+            raise ValueError("Illegal chunk pattern: %s" % rule) from e
 
 
 class ChunkRule(RegexpChunkRule):
@@ -402,7 +404,6 @@ class ChunkRule(RegexpChunkRule):
     """
 
     def __init__(self, tag_pattern, descr):
-
         """
         Construct a new ``ChunkRule``.
 
@@ -418,9 +419,9 @@ class ChunkRule(RegexpChunkRule):
         self._pattern = tag_pattern
         regexp = re.compile(
             "(?P<chunk>%s)%s"
-            % (tag_pattern2re_pattern(tag_pattern), ChunkString.IN_CHINK_PATTERN)
+            % (tag_pattern2re_pattern(tag_pattern), ChunkString.IN_STRIP_PATTERN)
         )
-        RegexpChunkRule.__init__(self, regexp, "{\g<chunk>}", descr)
+        RegexpChunkRule.__init__(self, regexp, r"{\g<chunk>}", descr)
 
     def __repr__(self):
         """
@@ -437,9 +438,9 @@ class ChunkRule(RegexpChunkRule):
         return "<ChunkRule: " + repr(self._pattern) + ">"
 
 
-class ChinkRule(RegexpChunkRule):
+class StripRule(RegexpChunkRule):
     """
-    A rule specifying how to remove chinks to a ``ChunkString``,
+    A rule specifying how to remove strips to a ``ChunkString``,
     using a matching tag pattern.  When applied to a
     ``ChunkString``, it will find any substring that matches this
     tag pattern and that is contained in a chunk, and remove it
@@ -448,7 +449,7 @@ class ChinkRule(RegexpChunkRule):
 
     def __init__(self, tag_pattern, descr):
         """
-        Construct a new ``ChinkRule``.
+        Construct a new ``StripRule``.
 
         :type tag_pattern: str
         :param tag_pattern: This rule's tag pattern.  When
@@ -462,16 +463,16 @@ class ChinkRule(RegexpChunkRule):
         """
         self._pattern = tag_pattern
         regexp = re.compile(
-            "(?P<chink>%s)%s"
+            "(?P<strip>%s)%s"
             % (tag_pattern2re_pattern(tag_pattern), ChunkString.IN_CHUNK_PATTERN)
         )
-        RegexpChunkRule.__init__(self, regexp, "}\g<chink>{", descr)
+        RegexpChunkRule.__init__(self, regexp, r"}\g<strip>{", descr)
 
     def __repr__(self):
         """
         Return a string representation of this rule.  It has the form::
 
-            <ChinkRule: '<IN|VB.*>'>
+            <StripRule: '<IN|VB.*>'>
 
         Note that this representation does not include the
         description string; that string can be accessed
@@ -479,7 +480,7 @@ class ChinkRule(RegexpChunkRule):
 
         :rtype: str
         """
-        return "<ChinkRule: " + repr(self._pattern) + ">"
+        return "<StripRule: " + repr(self._pattern) + ">"
 
 
 class UnChunkRule(RegexpChunkRule):
@@ -504,8 +505,8 @@ class UnChunkRule(RegexpChunkRule):
             of this rule.
         """
         self._pattern = tag_pattern
-        regexp = re.compile("\{(?P<chunk>%s)\}" % tag_pattern2re_pattern(tag_pattern))
-        RegexpChunkRule.__init__(self, regexp, "\g<chunk>", descr)
+        regexp = re.compile(r"\{(?P<chunk>%s)\}" % tag_pattern2re_pattern(tag_pattern))
+        RegexpChunkRule.__init__(self, regexp, r"\g<chunk>", descr)
 
     def __repr__(self):
         """
@@ -569,7 +570,7 @@ class MergeRule(RegexpChunkRule):
                 tag_pattern2re_pattern(right_tag_pattern),
             )
         )
-        RegexpChunkRule.__init__(self, regexp, "\g<left>", descr)
+        RegexpChunkRule.__init__(self, regexp, r"\g<left>", descr)
 
     def __repr__(self):
         """
@@ -650,7 +651,7 @@ class SplitRule(RegexpChunkRule):
         description string; that string can be accessed
         separately with the ``descr()`` method.
 
-       :rtype: str
+        :rtype: str
         """
         return (
             "<SplitRule: "
@@ -666,7 +667,7 @@ class ExpandLeftRule(RegexpChunkRule):
     A rule specifying how to expand chunks in a ``ChunkString`` to the left,
     using two matching tag patterns: a left pattern, and a right pattern.
     When applied to a ``ChunkString``, it will find any chunk whose beginning
-    matches right pattern, and immediately preceded by a chink whose
+    matches right pattern, and immediately preceded by a strip whose
     end matches left pattern.  It will then expand the chunk to incorporate
     the new material on the left.
     """
@@ -679,14 +680,14 @@ class ExpandLeftRule(RegexpChunkRule):
         :param right_tag_pattern: This rule's right tag
             pattern.  When applied to a ``ChunkString``, this
             rule will find any chunk whose beginning matches
-            ``right_tag_pattern``, and immediately preceded by a chink
+            ``right_tag_pattern``, and immediately preceded by a strip
             whose end matches this pattern.  It will
             then merge those two chunks into a single chunk.
         :type left_tag_pattern: str
         :param left_tag_pattern: This rule's left tag
             pattern.  When applied to a ``ChunkString``, this
             rule will find any chunk whose beginning matches
-            this pattern, and immediately preceded by a chink
+            this pattern, and immediately preceded by a strip
             whose end matches ``left_tag_pattern``.  It will
             then expand the chunk to incorporate the new material on the left.
 
@@ -702,13 +703,13 @@ class ExpandLeftRule(RegexpChunkRule):
         self._left_tag_pattern = left_tag_pattern
         self._right_tag_pattern = right_tag_pattern
         regexp = re.compile(
-            "(?P<left>%s)\{(?P<right>%s)"
+            r"(?P<left>%s)\{(?P<right>%s)"
             % (
                 tag_pattern2re_pattern(left_tag_pattern),
                 tag_pattern2re_pattern(right_tag_pattern),
             )
         )
-        RegexpChunkRule.__init__(self, regexp, "{\g<left>\g<right>", descr)
+        RegexpChunkRule.__init__(self, regexp, r"{\g<left>\g<right>", descr)
 
     def __repr__(self):
         """
@@ -737,7 +738,7 @@ class ExpandRightRule(RegexpChunkRule):
     right, using two matching tag patterns: a left pattern, and a
     right pattern.  When applied to a ``ChunkString``, it will find any
     chunk whose end matches left pattern, and immediately followed by
-    a chink whose beginning matches right pattern.  It will then
+    a strip whose beginning matches right pattern.  It will then
     expand the chunk to incorporate the new material on the right.
     """
 
@@ -749,14 +750,14 @@ class ExpandRightRule(RegexpChunkRule):
         :param right_tag_pattern: This rule's right tag
             pattern.  When applied to a ``ChunkString``, this
             rule will find any chunk whose end matches
-            ``left_tag_pattern``, and immediately followed by a chink
+            ``left_tag_pattern``, and immediately followed by a strip
             whose beginning matches this pattern.  It will
             then merge those two chunks into a single chunk.
         :type left_tag_pattern: str
         :param left_tag_pattern: This rule's left tag
             pattern.  When applied to a ``ChunkString``, this
             rule will find any chunk whose end matches
-            this pattern, and immediately followed by a chink
+            this pattern, and immediately followed by a strip
             whose beginning matches ``right_tag_pattern``.  It will
             then expand the chunk to incorporate the new material on the right.
 
@@ -772,13 +773,13 @@ class ExpandRightRule(RegexpChunkRule):
         self._left_tag_pattern = left_tag_pattern
         self._right_tag_pattern = right_tag_pattern
         regexp = re.compile(
-            "(?P<left>%s)\}(?P<right>%s)"
+            r"(?P<left>%s)\}(?P<right>%s)"
             % (
                 tag_pattern2re_pattern(left_tag_pattern),
                 tag_pattern2re_pattern(right_tag_pattern),
             )
         )
-        RegexpChunkRule.__init__(self, regexp, "\g<left>\g<right>}", descr)
+        RegexpChunkRule.__init__(self, regexp, r"\g<left>\g<right>}", descr)
 
     def __repr__(self):
         """
@@ -858,7 +859,7 @@ class ChunkRuleWithContext(RegexpChunkRule):
                 tag_pattern2re_pattern(left_context_tag_pattern),
                 tag_pattern2re_pattern(chunk_tag_pattern),
                 tag_pattern2re_pattern(right_context_tag_pattern),
-                ChunkString.IN_CHINK_PATTERN,
+                ChunkString.IN_STRIP_PATTERN,
             )
         )
         replacement = r"\g<left>{\g<chunk>}\g<right>"
@@ -876,21 +877,21 @@ class ChunkRuleWithContext(RegexpChunkRule):
 
         :rtype: str
         """
-        return "<ChunkRuleWithContext:  %r, %r, %r>" % (
+        return "<ChunkRuleWithContext:  {!r}, {!r}, {!r}>".format(
             self._left_context_tag_pattern,
             self._chunk_tag_pattern,
             self._right_context_tag_pattern,
         )
 
 
-##//////////////////////////////////////////////////////
-##  Tag Pattern Format Conversion
-##//////////////////////////////////////////////////////
+# //////////////////////////////////////////////////////
+# Tag Pattern Format Conversion
+# //////////////////////////////////////////////////////
 
 # this should probably be made more strict than it is -- e.g., it
 # currently accepts 'foo'.
 CHUNK_TAG_PATTERN = re.compile(
-    r"^((%s|<%s>)*)$" % ("([^\{\}<>]|\{\d+,?\}|\{\d*,\d+\})+", "[^\{\}<>]+")
+    r"^(({}|<{}>)*)$".format(r"([^\{\}<>]|\{\d+,?\}|\{\d*,\d+\})+", r"[^\{\}<>]+")
 )
 
 
@@ -905,8 +906,8 @@ def tag_pattern2re_pattern(tag_pattern):
           ``'<NN>+'`` matches one or more repetitions of ``'<NN>'``, not
           ``'<NN'`` followed by one or more repetitions of ``'>'``.
         - Whitespace in tag patterns is ignored.  So
-          ``'<DT> | <NN>'`` is equivalant to ``'<DT>|<NN>'``
-        - In tag patterns, ``'.'`` is equivalant to ``'[^{}<>]'``; so
+          ``'<DT> | <NN>'`` is equivalent to ``'<DT>|<NN>'``
+        - In tag patterns, ``'.'`` is equivalent to ``'[^{}<>]'``; so
           ``'<NN.*>'`` matches any single tag starting with ``'NN'``.
 
     In particular, ``tag_pattern2re_pattern`` performs the following
@@ -958,9 +959,9 @@ def tag_pattern2re_pattern(tag_pattern):
     return tag_pattern
 
 
-##//////////////////////////////////////////////////////
-##  RegexpChunkParser
-##//////////////////////////////////////////////////////
+# //////////////////////////////////////////////////////
+# RegexpChunkParser
+# //////////////////////////////////////////////////////
 
 
 class RegexpChunkParser(ChunkParserI):
@@ -973,7 +974,7 @@ class RegexpChunkParser(ChunkParserI):
     expression matching and substitution.
 
     The ``RegexpChunkRule`` class and its subclasses (``ChunkRule``,
-    ``ChinkRule``, ``UnChunkRule``, ``MergeRule``, and ``SplitRule``)
+    ``StripRule``, ``UnChunkRule``, ``MergeRule``, and ``SplitRule``)
     define the rules that are used by ``RegexpChunkParser``.  Each rule
     defines an ``apply()`` method, which modifies the chunking encoded
     by a given ``ChunkString``.
@@ -1056,7 +1057,7 @@ class RegexpChunkParser(ChunkParserI):
         :param trace: The level of tracing that should be used when
             parsing a text.  ``0`` will generate no tracing output;
             ``1`` will generate normal tracing output; and ``2`` or
-            highter will generate verbose tracing output.  This value
+            higher will generate verbose tracing output.  This value
             overrides the trace level value that was given to the
             constructor.
         :rtype: Tree
@@ -1124,13 +1125,13 @@ class RegexpChunkParser(ChunkParserI):
         return s[:-1]
 
 
-##//////////////////////////////////////////////////////
-##  Chunk Grammar
-##//////////////////////////////////////////////////////
+# //////////////////////////////////////////////////////
+# Chunk Grammar
+# //////////////////////////////////////////////////////
 
 
 class RegexpParser(ChunkParserI):
-    """
+    r"""
     A grammar based chunk parser.  ``chunk.RegexpParser`` uses a set of
     regular expression patterns to specify the behavior of the parser.
     The chunking of the text is encoded using a ``ChunkString``, and
@@ -1142,7 +1143,7 @@ class RegexpParser(ChunkParserI):
 
      NP:
        {<DT|JJ>}          # chunk determiners and adjectives
-       }<[\.VI].*>+{      # chink any tag beginning with V, I, or .
+       }<[\.VI].*>+{      # strip any tag beginning with V, I, or .
        <.*>}{<DT>         # split a chunk at a determiner
        <DT|JJ>{}<NN.*>    # merge chunk ending with det/adj
                           # with one starting with a noun
@@ -1202,8 +1203,8 @@ class RegexpParser(ChunkParserI):
             )
             try:
                 grammar = list(grammar)
-            except:
-                raise TypeError(type_err)
+            except BaseException as e:
+                raise TypeError(type_err) from e
             for elt in grammar:
                 if not isinstance(elt, RegexpChunkParser):
                     raise TypeError(type_err)
@@ -1262,7 +1263,7 @@ class RegexpParser(ChunkParserI):
         :param trace: The level of tracing that should be used when
             parsing a text.  ``0`` will generate no tracing output;
             ``1`` will generate normal tracing output; and ``2`` or
-            highter will generate verbose tracing output.  This value
+            higher will generate verbose tracing output.  This value
             overrides the trace level value that was given to the
             constructor.
         :return: the chunked output.
@@ -1295,9 +1296,9 @@ class RegexpParser(ChunkParserI):
         return s[:-1]
 
 
-##//////////////////////////////////////////////////////
-##  Demonstration code
-##//////////////////////////////////////////////////////
+# //////////////////////////////////////////////////////
+# Demonstration code
+# //////////////////////////////////////////////////////
 
 
 def demo_eval(chunkparser, text):
@@ -1336,7 +1337,7 @@ def demo_eval(chunkparser, text):
 
     print("/" + ("=" * 75) + "\\")
     print("Scoring", chunkparser)
-    print(("-" * 77))
+    print("-" * 77)
     print("Precision: %5.1f%%" % (chunkscore.precision() * 100), " " * 4, end=" ")
     print("Recall: %5.1f%%" % (chunkscore.recall() * 100), " " * 6, end=" ")
     print("F-Measure: %5.1f%%" % (chunkscore.f_measure() * 100))
@@ -1370,7 +1371,7 @@ def demo():
     and strategies.
     """
 
-    from nltk import chunk, Tree
+    from nltk import Tree, chunk
 
     text = """\
     [ the/DT little/JJ cat/NN ] sat/VBD on/IN [ the/DT mat/NN ] ./.
@@ -1410,7 +1411,7 @@ def demo():
 
     grammar = r"""
     NP: {<.*>*}             # start by chunking everything
-        }<[\.VI].*>+{       # chink any verbs, prepositions or periods
+        }<[\.VI].*>+{       # strip any verbs, prepositions or periods
         <.*>}{<DT>          # separate on determiners
     PP: {<IN><NP>}          # PP = preposition + noun phrase
     VP: {<VB.*><NP|PP>*}    # VP = verb words + NPs and PPs
@@ -1445,7 +1446,7 @@ def demo():
 
     grammar = r"""
     NP: {<.*>*}             # start by chunking everything
-        }<[\.VI].*>+{       # chink any verbs, prepositions or periods
+        }<[\.VI].*>+{       # strip any verbs, prepositions or periods
         <.*>}{<DT>          # separate on determiners
     PP: {<IN><NP>}          # PP = preposition + noun phrase
     VP: {<VB.*><NP|PP>*}    # VP = verb words + NPs and PPs
