@@ -1,8 +1,8 @@
 # Natural Language Toolkit: Carnegie Mellon Pronouncing Dictionary Corpus Reader
 #
-# Copyright (C) 2001-2018 NLTK Project
+# Copyright (C) 2001-2022 NLTK Project
 # Author: Steven Bird <stevenbird1@gmail.com>
-# URL: <http://nltk.org/>
+# URL: <https://www.nltk.org/>
 # For license information, see LICENSE.TXT
 
 """
@@ -45,34 +45,23 @@ Y       yield   Y IY L D       Z       zee     Z IY
 ZH      seizure S IY ZH ER
 """
 
-import codecs
-
-from six import string_types
-
-from nltk import compat
+from nltk.corpus.reader.api import *
+from nltk.corpus.reader.util import *
 from nltk.util import Index
 
-from nltk.corpus.reader.util import *
-from nltk.corpus.reader.api import *
 
 class CMUDictCorpusReader(CorpusReader):
     def entries(self):
         """
         :return: the cmudict lexicon as a list of entries
-        containing (word, transcriptions) tuples.
+            containing (word, transcriptions) tuples.
         """
-        return concat([StreamBackedCorpusView(fileid, read_cmudict_block,
-                                              encoding=enc)
-                       for fileid, enc in self.abspaths(None, True)])
-
-    def raw(self):
-        """
-        :return: the cmudict lexicon as a raw string.
-        """
-        fileids = self._fileids
-        if isinstance(fileids, string_types):
-            fileids = [fileids]
-        return concat([self.open(f).read() for f in fileids])
+        return concat(
+            [
+                StreamBackedCorpusView(fileid, read_cmudict_block, encoding=enc)
+                for fileid, enc in self.abspaths(None, True)
+            ]
+        )
 
     def words(self):
         """
@@ -83,15 +72,17 @@ class CMUDictCorpusReader(CorpusReader):
     def dict(self):
         """
         :return: the cmudict lexicon as a dictionary, whose keys are
-        lowercase words and whose values are lists of pronunciations.
+            lowercase words and whose values are lists of pronunciations.
         """
         return dict(Index(self.entries()))
 
+
 def read_cmudict_block(stream):
     entries = []
-    while len(entries) < 100: # Read 100 at a time.
+    while len(entries) < 100:  # Read 100 at a time.
         line = stream.readline()
-        if line == '': return entries # end of file.
+        if line == "":
+            return entries  # end of file.
         pieces = line.split()
-        entries.append( (pieces[0].lower(), pieces[2:]) )
+        entries.append((pieces[0].lower(), pieces[2:]))
     return entries
