@@ -1,34 +1,28 @@
 # Natural Language Toolkit: Corpus Reader Utilities
 #
-# Copyright (C) 2001-2019 NLTK Project
+# Copyright (C) 2001-2022 NLTK Project
 # Author: Steven Bird <stevenbird1@gmail.com>
 #         Edward Loper <edloper@gmail.com>
-# URL: <http://nltk.org/>
+# URL: <https://www.nltk.org/>
 # For license information, see LICENSE.TXT
 
-import os
 import bisect
+import os
+import pickle
 import re
 import tempfile
 from functools import reduce
+from xml.etree import ElementTree
 
-try:
-    import cPickle as pickle
-except ImportError:
-    import pickle
-
-try:  # Use the c version of ElementTree, which is faster, if possible.
-    from xml.etree import cElementTree as ElementTree
-except ImportError:
-    from xml.etree import ElementTree
-
-from six import string_types, text_type
-
-from nltk.tokenize import wordpunct_tokenize
+from nltk.data import (
+    FileSystemPathPointer,
+    PathPointer,
+    SeekableUnicodeStreamReader,
+    ZipFilePathPointer,
+)
 from nltk.internals import slice_bounds
-from nltk.data import PathPointer, FileSystemPathPointer, ZipFilePathPointer
-from nltk.data import SeekableUnicodeStreamReader
-from nltk.util import AbstractLazySequence, LazySubsequence, LazyConcatenation, py25
+from nltk.tokenize import wordpunct_tokenize
+from nltk.util import AbstractLazySequence, LazyConcatenation, LazySubsequence
 
 ######################################################################
 # { Corpus View
@@ -128,7 +122,7 @@ class StreamBackedCorpusView(AbstractLazySequence):
        block; and tokens is a list of the tokens in the block.
     """
 
-    def __init__(self, fileid, block_reader=None, startpos=0, encoding='utf8'):
+    def __init__(self, fileid, block_reader=None, startpos=0, encoding="utf8"):
         """
         Create a new corpus view, based on the file ``fileid``, and
         read with ``block_reader``.  See the class documentation
@@ -180,7 +174,7 @@ class StreamBackedCorpusView(AbstractLazySequence):
             else:
                 self._eofpos = os.stat(self._fileid).st_size
         except Exception as exc:
-            raise ValueError('Unable to open or access %r -- %s' % (fileid, exc))
+            raise ValueError(f"Unable to open or access {fileid!r} -- {exc}") from exc
 
         # Maintain a cache of the most recently read block, to
         # increase efficiency of random access.
@@ -203,7 +197,7 @@ class StreamBackedCorpusView(AbstractLazySequence):
         :param stream: an input stream
         :type stream: stream
         """
-        raise NotImplementedError('Abstract Method')
+        raise NotImplementedError("Abstract Method")
 
     def _open(self):
         """
@@ -215,10 +209,10 @@ class StreamBackedCorpusView(AbstractLazySequence):
             self._stream = self._fileid.open(self._encoding)
         elif self._encoding:
             self._stream = SeekableUnicodeStreamReader(
-                open(self._fileid, 'rb'), self._encoding
+                open(self._fileid, "rb"), self._encoding
             )
         else:
-            self._stream = open(self._fileid, 'rb')
+            self._stream = open(self._fileid, "rb")
 
     def close(self):
         """
@@ -233,6 +227,12 @@ class StreamBackedCorpusView(AbstractLazySequence):
             self._stream.close()
         self._stream = None
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        self.close()
+
     def __len__(self):
         if self._len is None:
             # iterate_from() sets self._len when it reaches the end
@@ -255,7 +255,7 @@ class StreamBackedCorpusView(AbstractLazySequence):
             if i < 0:
                 i += len(self)
             if i < 0:
-                raise IndexError('index out of range')
+                raise IndexError("index out of range")
             # Check if it's in the cache.
             offset = self._cache[0]
             if offset <= i < self._cache[1]:
@@ -263,8 +263,8 @@ class StreamBackedCorpusView(AbstractLazySequence):
             # Use iterate_from to extract it.
             try:
                 return next(self.iterate_from(i))
-            except StopIteration:
-                raise IndexError('index out of range')
+            except StopIteration as e:
+                raise IndexError("index out of range") from e
 
     # If we wanted to be thread-safe, then this method would need to
     # do some locking.
@@ -305,14 +305,16 @@ class StreamBackedCorpusView(AbstractLazySequence):
             self._current_blocknum = block_index
             tokens = self.read_block(self._stream)
             assert isinstance(tokens, (tuple, list, AbstractLazySequence)), (
-                'block reader %s() should return list or tuple.'
+                "block reader %s() should return list or tuple."
                 % self.read_block.__name__
             )
             num_toks = len(tokens)
             new_filepos = self._stream.tell()
-            assert new_filepos > filepos, (
-                'block reader %s() should consume at least 1 byte (filepos=%d)'
-                % (self.read_block.__name__, filepos)
+            assert (
+                new_filepos > filepos
+            ), "block reader %s() should consume at least 1 byte (filepos=%d)" % (
+                self.read_block.__name__,
+                filepos,
             )
 
             # Update our cache.
@@ -330,10 +332,10 @@ class StreamBackedCorpusView(AbstractLazySequence):
                     # Check for consistency:
                     assert (
                         new_filepos == self._filepos[block_index]
-                    ), 'inconsistent block reader (num chars read)'
+                    ), "inconsistent block reader (num chars read)"
                     assert (
                         toknum + num_toks == self._toknum[block_index]
-                    ), 'inconsistent block reader (num tokens returned)'
+                    ), "inconsistent block reader (num tokens returned)"
 
             # If we reached the end of the file, then update self._len
             if new_filepos == self._eofpos:
@@ -419,8 +421,7 @@ class ConcatenatedCorpusView(AbstractLazySequence):
                 self._open_piece = piece
 
             # Get everything we can from this piece.
-            for tok in piece.iterate_from(max(0, start_tok - offset)):
-                yield tok
+            yield from piece.iterate_from(max(0, start_tok - offset))
 
             # Update the offset table.
             if piecenum + 1 == len(self._offsets):
@@ -440,13 +441,13 @@ def concat(docs):
     if len(docs) == 1:
         return docs[0]
     if len(docs) == 0:
-        raise ValueError('concat() expects at least one object!')
+        raise ValueError("concat() expects at least one object!")
 
-    types = set(d.__class__ for d in docs)
+    types = {d.__class__ for d in docs}
 
     # If they're all strings, use string concatenation.
-    if all(isinstance(doc, string_types) for doc in docs):
-        return ''.join(docs)
+    if all(isinstance(doc, str) for doc in docs):
+        return "".join(docs)
 
     # If they're all corpus views, then use ConcatenatedCorpusView.
     for typ in types:
@@ -473,7 +474,7 @@ def concat(docs):
             return reduce((lambda a, b: a + b), docs, ())
 
         if ElementTree.iselement(typ):
-            xmltree = ElementTree.Element('documents')
+            xmltree = ElementTree.Element("documents")
             for doc in docs:
                 xmltree.append(doc)
             return xmltree
@@ -534,18 +535,18 @@ class PickleCorpusView(StreamBackedCorpusView):
         fileid.  (This method is called whenever a
         ``PickledCorpusView`` is garbage-collected.
         """
-        if getattr(self, '_delete_on_gc'):
+        if getattr(self, "_delete_on_gc"):
             if os.path.exists(self._fileid):
                 try:
                     os.remove(self._fileid)
-                except (OSError, IOError):
+                except OSError:
                     pass
         self.__dict__.clear()  # make the garbage collector's job easier
 
     @classmethod
     def write(cls, sequence, output_file):
-        if isinstance(output_file, string_types):
-            output_file = open(output_file, 'wb')
+        if isinstance(output_file, str):
+            output_file = open(output_file, "wb")
         for item in sequence:
             pickle.dump(item, output_file, cls.PROTOCOL)
 
@@ -560,13 +561,13 @@ class PickleCorpusView(StreamBackedCorpusView):
             deleted whenever this object gets garbage-collected.
         """
         try:
-            fd, output_file_name = tempfile.mkstemp('.pcv', 'nltk-')
-            output_file = os.fdopen(fd, 'wb')
+            fd, output_file_name = tempfile.mkstemp(".pcv", "nltk-")
+            output_file = os.fdopen(fd, "wb")
             cls.write(sequence, output_file)
             output_file.close()
             return PickleCorpusView(output_file_name, delete_on_gc)
-        except (OSError, IOError) as e:
-            raise ValueError('Error while creating temp file: %s' % e)
+        except OSError as e:
+            raise ValueError("Error while creating temp file: %s" % e) from e
 
 
 ######################################################################
@@ -594,12 +595,12 @@ def read_line_block(stream):
         line = stream.readline()
         if not line:
             return toks
-        toks.append(line.rstrip('\n'))
+        toks.append(line.rstrip("\n"))
     return toks
 
 
 def read_blankline_block(stream):
-    s = ''
+    s = ""
     while True:
         line = stream.readline()
         # End of file:
@@ -618,10 +619,10 @@ def read_blankline_block(stream):
 
 
 def read_alignedsent_block(stream):
-    s = ''
+    s = ""
     while True:
         line = stream.readline()
-        if line[0] == '=' or line[0] == '\n' or line[:2] == '\r\n':
+        if line[0] == "=" or line[0] == "\n" or line[:2] == "\r\n":
             continue
         # End of file:
         if not line:
@@ -632,7 +633,7 @@ def read_alignedsent_block(stream):
         # Other line:
         else:
             s += line
-            if re.match('^\d+-\d+', line) is not None:
+            if re.match(r"^\d+-\d+", line) is not None:
                 return [s]
 
 
@@ -658,15 +659,15 @@ def read_regexp_block(stream, start_re, end_re=None):
         line = stream.readline()
         # End of file:
         if not line:
-            return [''.join(lines)]
+            return ["".join(lines)]
         # End of token:
         if end_re is not None and re.match(end_re, line):
-            return [''.join(lines)]
+            return ["".join(lines)]
         # Start of new token: backup to just before it starts, and
         # return the token we've already collected.
         if end_re is None and re.match(start_re, line):
             stream.seek(oldpos)
-            return [''.join(lines)]
+            return ["".join(lines)]
         # Anything else is part of the token.
         lines.append(line)
 
@@ -692,20 +693,20 @@ def read_sexpr_block(stream, block_size=16384, comment_char=None):
     """
     start = stream.tell()
     block = stream.read(block_size)
-    encoding = getattr(stream, 'encoding', None)
-    assert encoding is not None or isinstance(block, text_type)
-    if encoding not in (None, 'utf-8'):
+    encoding = getattr(stream, "encoding", None)
+    assert encoding is not None or isinstance(block, str)
+    if encoding not in (None, "utf-8"):
         import warnings
 
         warnings.warn(
-            'Parsing may fail, depending on the properties '
-            'of the %s encoding!' % encoding
+            "Parsing may fail, depending on the properties "
+            "of the %s encoding!" % encoding
         )
         # (e.g., the utf-16 encoding does not work because it insists
         # on adding BOMs to the beginning of encoded strings.)
 
     if comment_char:
-        COMMENT = re.compile('(?m)^%s.*$' % re.escape(comment_char))
+        COMMENT = re.compile("(?m)^%s.*$" % re.escape(comment_char))
     while True:
         try:
             # If we're stripping comments, then make sure our block ends
@@ -718,7 +719,7 @@ def read_sexpr_block(stream, block_size=16384, comment_char=None):
             # Read the block.
             tokens, offset = _parse_sexpr_block(block)
             # Skip whitespace
-            offset = re.compile(r'\s*').search(block, offset).end()
+            offset = re.compile(r"\s*").search(block, offset).end()
 
             # Move to the end position.
             if encoding is None:
@@ -729,7 +730,7 @@ def read_sexpr_block(stream, block_size=16384, comment_char=None):
             # Return the list of tokens we processed
             return tokens
         except ValueError as e:
-            if e.args[0] == 'Block too small':
+            if e.args[0] == "Block too small":
                 next_block = stream.read(block_size)
                 if next_block:
                     block += next_block
@@ -744,7 +745,7 @@ def read_sexpr_block(stream, block_size=16384, comment_char=None):
 def _sub_space(m):
     """Helper function: given a regexp match, return a string of
     spaces that's the same length as the matched string."""
-    return ' ' * (m.end() - m.start())
+    return " " * (m.end() - m.start())
 
 
 def _parse_sexpr_block(block):
@@ -752,27 +753,27 @@ def _parse_sexpr_block(block):
     start = end = 0
 
     while end < len(block):
-        m = re.compile(r'\S').search(block, end)
+        m = re.compile(r"\S").search(block, end)
         if not m:
             return tokens, end
 
         start = m.start()
 
         # Case 1: sexpr is not parenthesized.
-        if m.group() != '(':
-            m2 = re.compile(r'[\s(]').search(block, start)
+        if m.group() != "(":
+            m2 = re.compile(r"[\s(]").search(block, start)
             if m2:
                 end = m2.start()
             else:
                 if tokens:
                     return tokens, end
-                raise ValueError('Block too small')
+                raise ValueError("Block too small")
 
         # Case 2: parenthesized sexpr.
         else:
             nesting = 0
-            for m in re.compile(r'[()]').finditer(block, start):
-                if m.group() == '(':
+            for m in re.compile(r"[()]").finditer(block, start):
+                if m.group() == "(":
                     nesting += 1
                 else:
                     nesting -= 1
@@ -782,7 +783,7 @@ def _parse_sexpr_block(block):
             else:
                 if tokens:
                     return tokens, end
-                raise ValueError('Block too small')
+                raise ValueError("Block too small")
 
         tokens.append(block[start:end])
 
@@ -796,8 +797,8 @@ def _parse_sexpr_block(block):
 
 def find_corpus_fileids(root, regexp):
     if not isinstance(root, PathPointer):
-        raise TypeError('find_corpus_fileids: expected a PathPointer')
-    regexp += '$'
+        raise TypeError("find_corpus_fileids: expected a PathPointer")
+    regexp += "$"
 
     # Find fileids in a zipfile: scan the zipfile's namelist.  Filter
     # out entries that end in '/' -- they're directories.
@@ -805,7 +806,7 @@ def find_corpus_fileids(root, regexp):
         fileids = [
             name[len(root.entry) :]
             for name in root.zipfile.namelist()
-            if not name.endswith('/')
+            if not name.endswith("/")
         ]
         items = [name for name in fileids if re.match(regexp, name)]
         return sorted(items)
@@ -814,20 +815,16 @@ def find_corpus_fileids(root, regexp):
     # or symlinked) subdirectories, and match paths against the regexp.
     elif isinstance(root, FileSystemPathPointer):
         items = []
-        # workaround for py25 which doesn't support followlinks
-        kwargs = {}
-        if not py25():
-            kwargs = {'followlinks': True}
-        for dirname, subdirs, fileids in os.walk(root.path, **kwargs):
-            prefix = ''.join('%s/' % p for p in _path_from(root.path, dirname))
+        for dirname, subdirs, fileids in os.walk(root.path):
+            prefix = "".join("%s/" % p for p in _path_from(root.path, dirname))
             items += [
                 prefix + fileid
                 for fileid in fileids
                 if re.match(regexp, prefix + fileid)
             ]
             # Don't visit svn directories:
-            if '.svn' in subdirs:
-                subdirs.remove('.svn')
+            if ".svn" in subdirs:
+                subdirs.remove(".svn")
         return sorted(items)
 
     else:
@@ -835,7 +832,7 @@ def find_corpus_fileids(root, regexp):
 
 
 def _path_from(parent, child):
-    if os.path.split(parent)[1] == '':
+    if os.path.split(parent)[1] == "":
         parent = os.path.split(parent)[0]
     path = []
     while parent != child:
@@ -852,15 +849,15 @@ def _path_from(parent, child):
 
 def tagged_treebank_para_block_reader(stream):
     # Read the next paragraph.
-    para = ''
+    para = ""
     while True:
         line = stream.readline()
         # End of paragraph:
-        if re.match('======+\s*$', line):
+        if re.match(r"======+\s*$", line):
             if para.strip():
                 return [para]
         # End of file:
-        elif line == '':
+        elif line == "":
             if para.strip():
                 return [para]
             else:
