@@ -1,8 +1,8 @@
 # Natural Language Toolkit: Agreement Metrics
 #
-# Copyright (C) 2001-2019 NLTK Project
+# Copyright (C) 2001-2022 NLTK Project
 # Author: Tom Lippincott <tom@cs.columbia.edu>
-# URL: <http://nltk.org/>
+# URL: <https://www.nltk.org/>
 # For license information, see LICENSE.TXT
 #
 
@@ -68,25 +68,19 @@ Expected results from the Artstein and Poesio survey paper:
     1.0
 
 """
-from __future__ import print_function, unicode_literals, division
 
 import logging
 from itertools import groupby
 from operator import itemgetter
 
-from six import iteritems
-
-from nltk.probability import FreqDist, ConditionalFreqDist
 from nltk.internals import deprecated
-from nltk.compat import python_2_unicode_compatible
-
 from nltk.metrics.distance import binary_distance
+from nltk.probability import ConditionalFreqDist, FreqDist
 
 log = logging.getLogger(__name__)
 
 
-@python_2_unicode_compatible
-class AnnotationTask(object):
+class AnnotationTask:
     """Represents an annotation task, i.e. people assign labels to items.
 
     Notation tries to match notation in Artstein and Poesio (2007).
@@ -104,11 +98,11 @@ class AnnotationTask(object):
 
         The data argument can be None (to create an empty annotation task) or a sequence of 3-tuples,
         each representing a coder's labeling of an item:
-            (coder,item,label)
+        ``(coder,item,label)``
 
         The distance argument is a function taking two arguments (labels) and producing a numerical distance.
         The distance from a label to itself should be zero:
-            distance(l,l) = 0
+        ``distance(l,l) = 0``
         """
         self.distance = distance
         self.I = set()
@@ -122,7 +116,7 @@ class AnnotationTask(object):
         return "\r\n".join(
             map(
                 lambda x: "%s\t%s\t%s"
-                % (x['coder'], x['item'].replace('_', "\t"), ",".join(x['labels'])),
+                % (x["coder"], x["item"].replace("_", "\t"), ",".join(x["labels"])),
                 self.data,
             )
         )
@@ -137,46 +131,39 @@ class AnnotationTask(object):
             self.C.add(coder)
             self.K.add(labels)
             self.I.add(item)
-            self.data.append({'coder': coder, 'labels': labels, 'item': item})
+            self.data.append({"coder": coder, "labels": labels, "item": item})
 
     def agr(self, cA, cB, i, data=None):
-        """Agreement between two coders on a given item
-
-        """
+        """Agreement between two coders on a given item"""
         data = data or self.data
         # cfedermann: we don't know what combination of coder/item will come
         # first in x; to avoid StopIteration problems due to assuming an order
         # cA,cB, we allow either for k1 and then look up the missing as k2.
-        k1 = next((x for x in data if x['coder'] in (cA, cB) and x['item'] == i))
-        if k1['coder'] == cA:
-            k2 = next((x for x in data if x['coder'] == cB and x['item'] == i))
+        k1 = next(x for x in data if x["coder"] in (cA, cB) and x["item"] == i)
+        if k1["coder"] == cA:
+            k2 = next(x for x in data if x["coder"] == cB and x["item"] == i)
         else:
-            k2 = next((x for x in data if x['coder'] == cA and x['item'] == i))
+            k2 = next(x for x in data if x["coder"] == cA and x["item"] == i)
 
-        ret = 1.0 - float(self.distance(k1['labels'], k2['labels']))
+        ret = 1.0 - float(self.distance(k1["labels"], k2["labels"]))
         log.debug("Observed agreement between %s and %s on %s: %f", cA, cB, i, ret)
         log.debug(
-            "Distance between \"%r\" and \"%r\": %f",
-            k1['labels'],
-            k2['labels'],
-            1.0 - ret,
+            'Distance between "%r" and "%r": %f', k1["labels"], k2["labels"], 1.0 - ret
         )
         return ret
 
     def Nk(self, k):
-        return float(sum(1 for x in self.data if x['labels'] == k))
+        return float(sum(1 for x in self.data if x["labels"] == k))
 
     def Nik(self, i, k):
-        return float(sum(1 for x in self.data if x['item'] == i and x['labels'] == k))
+        return float(sum(1 for x in self.data if x["item"] == i and x["labels"] == k))
 
     def Nck(self, c, k):
-        return float(sum(1 for x in self.data if x['coder'] == c and x['labels'] == k))
+        return float(sum(1 for x in self.data if x["coder"] == c and x["labels"] == k))
 
-    @deprecated('Use Nk, Nik or Nck instead')
+    @deprecated("Use Nk, Nik or Nck instead")
     def N(self, k=None, i=None, c=None):
-        """Implements the "n-notation" used in Artstein and Poesio (2007)
-
-        """
+        """Implements the "n-notation" used in Artstein and Poesio (2007)"""
         if k is not None and i is None and c is None:
             ret = self.Nk(k)
         elif k is not None and i is not None and c is None:
@@ -185,7 +172,7 @@ class AnnotationTask(object):
             ret = self.Nck(c, k)
         else:
             raise ValueError(
-                "You must pass either i or c, not both! (k=%r,i=%r,c=%r)" % (k, i, c)
+                f"You must pass either i or c, not both! (k={k!r},i={i!r},c={c!r})"
             )
         log.debug("Count on N[%s,%s,%s]: %d", k, i, c, ret)
         return ret
@@ -195,11 +182,9 @@ class AnnotationTask(object):
         return groupby(sorted(data, key=itemgetter(field)), itemgetter(field))
 
     def Ao(self, cA, cB):
-        """Observed agreement between two coders on all items.
-
-        """
+        """Observed agreement between two coders on all items."""
         data = self._grouped_data(
-            'item', (x for x in self.data if x['coder'] in (cA, cB))
+            "item", (x for x in self.data if x["coder"] in (cA, cB))
         )
         ret = sum(self.agr(cA, cB, item, item_data) for item, item_data in data) / len(
             self.I
@@ -223,31 +208,25 @@ class AnnotationTask(object):
         return ret
 
     def avg_Ao(self):
-        """Average observed agreement across all coders and items.
-
-        """
+        """Average observed agreement across all coders and items."""
         ret = self._pairwise_average(self.Ao)
         log.debug("Average observed agreement: %f", ret)
         return ret
 
     def Do_Kw_pairwise(self, cA, cB, max_distance=1.0):
-        """The observed disagreement for the weighted kappa coefficient.
-
-        """
+        """The observed disagreement for the weighted kappa coefficient."""
         total = 0.0
-        data = (x for x in self.data if x['coder'] in (cA, cB))
-        for i, itemdata in self._grouped_data('item', data):
+        data = (x for x in self.data if x["coder"] in (cA, cB))
+        for i, itemdata in self._grouped_data("item", data):
             # we should have two items; distance doesn't care which comes first
-            total += self.distance(next(itemdata)['labels'], next(itemdata)['labels'])
+            total += self.distance(next(itemdata)["labels"], next(itemdata)["labels"])
 
         ret = total / (len(self.I) * max_distance)
         log.debug("Observed disagreement between %s and %s: %f", cA, cB, ret)
         return ret
 
     def Do_Kw(self, max_distance=1.0):
-        """Averaged over all labelers
-
-        """
+        """Averaged over all labelers"""
         ret = self._pairwise_average(
             lambda cA, cB: self.Do_Kw_pairwise(cA, cB, max_distance)
         )
@@ -256,9 +235,7 @@ class AnnotationTask(object):
 
     # Agreement Coefficients
     def S(self):
-        """Bennett, Albert and Goldstein 1954
-
-        """
+        """Bennett, Albert and Goldstein 1954"""
         Ae = 1.0 / len(self.K)
         ret = (self.avg_Ao() - Ae) / (1.0 - Ae)
         return ret
@@ -269,8 +246,8 @@ class AnnotationTask(object):
 
         """
         total = 0.0
-        label_freqs = FreqDist(x['labels'] for x in self.data)
-        for k, f in iteritems(label_freqs):
+        label_freqs = FreqDist(x["labels"] for x in self.data)
+        for k, f in label_freqs.items():
             total += f ** 2
         Ae = total / ((len(self.I) * len(self.C)) ** 2)
         return (self.avg_Ao() - Ae) / (1 - Ae)
@@ -278,15 +255,13 @@ class AnnotationTask(object):
     def Ae_kappa(self, cA, cB):
         Ae = 0.0
         nitems = float(len(self.I))
-        label_freqs = ConditionalFreqDist((x['labels'], x['coder']) for x in self.data)
+        label_freqs = ConditionalFreqDist((x["labels"], x["coder"]) for x in self.data)
         for k in label_freqs.conditions():
             Ae += (label_freqs[k][cA] / nitems) * (label_freqs[k][cB] / nitems)
         return Ae
 
     def kappa_pairwise(self, cA, cB):
-        """
-
-        """
+        """ """
         Ae = self.Ae_kappa(cA, cB)
         ret = (self.Ao(cA, cB) - Ae) / (1.0 - Ae)
         log.debug("Expected agreement between %s and %s: %f", cA, cB, Ae)
@@ -310,20 +285,18 @@ class AnnotationTask(object):
     def Disagreement(self, label_freqs):
         total_labels = sum(label_freqs.values())
         pairs = 0.0
-        for j, nj in iteritems(label_freqs):
-            for l, nl in iteritems(label_freqs):
+        for j, nj in label_freqs.items():
+            for l, nl in label_freqs.items():
                 pairs += float(nj * nl) * self.distance(l, j)
         return 1.0 * pairs / (total_labels * (total_labels - 1))
 
     def alpha(self):
-        """Krippendorff 1980
-
-        """
+        """Krippendorff 1980"""
         # check for degenerate cases
         if len(self.K) == 0:
             raise ValueError("Cannot calculate alpha, no data present!")
         if len(self.K) == 1:
-            log.debug("Only one annotation value, allpha returning 1.")
+            log.debug("Only one annotation value, alpha returning 1.")
             return 1
         if len(self.C) == 1 and len(self.I) == 1:
             raise ValueError("Cannot calculate alpha, only one coder and item present!")
@@ -332,9 +305,9 @@ class AnnotationTask(object):
         total_ratings = 0
         all_valid_labels_freq = FreqDist([])
 
-        total_do = 0.0 # Total observed disagreement for all items.
-        for i, itemdata in self._grouped_data('item'):
-            label_freqs = FreqDist(x['labels'] for x in itemdata)
+        total_do = 0.0  # Total observed disagreement for all items.
+        for i, itemdata in self._grouped_data("item"):
+            label_freqs = FreqDist(x["labels"] for x in itemdata)
             labels_count = sum(label_freqs.values())
             if labels_count < 2:
                 # Ignore the item.
@@ -344,18 +317,16 @@ class AnnotationTask(object):
 
         do = total_do / sum(all_valid_labels_freq.values())
 
-        de = self.Disagreement(all_valid_labels_freq) # Expected disagreement.
+        de = self.Disagreement(all_valid_labels_freq)  # Expected disagreement.
         k_alpha = 1.0 - do / de
 
         return k_alpha
 
     def weighted_kappa_pairwise(self, cA, cB, max_distance=1.0):
-        """Cohen 1968
-
-        """
+        """Cohen 1968"""
         total = 0.0
         label_freqs = ConditionalFreqDist(
-            (x['coder'], x['labels']) for x in self.data if x['coder'] in (cA, cB)
+            (x["coder"], x["labels"]) for x in self.data if x["coder"] in (cA, cB)
         )
         for j in self.K:
             for l in self.K:
@@ -367,18 +338,17 @@ class AnnotationTask(object):
         return ret
 
     def weighted_kappa(self, max_distance=1.0):
-        """Cohen 1968
-
-        """
+        """Cohen 1968"""
         return self._pairwise_average(
             lambda cA, cB: self.weighted_kappa_pairwise(cA, cB, max_distance)
         )
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
 
-    import re
     import optparse
+    import re
+
     from nltk.metrics import distance
 
     # process command-line arguments
@@ -423,7 +393,7 @@ if __name__ == '__main__':
         "-v",
         "--verbose",
         dest="verbose",
-        default='0',
+        default="0",
         help="how much debugging to print on stderr (0-4)",
     )
     parser.add_option(
@@ -465,7 +435,7 @@ if __name__ == '__main__':
 
     # read in data from the specified file
     data = []
-    with open(options.file, 'r') as infile:
+    with open(options.file) as infile:
         for l in infile:
             toks = l.split(options.columnsep)
             coder, object_, labels = (
