@@ -1,26 +1,23 @@
-# -*- coding: utf-8 -*-
 # Natural Language Toolkit: Interface to the Stanford Parser
 #
-# Copyright (C) 2001-2020 NLTK Project
+# Copyright (C) 2001-2022 NLTK Project
 # Author: Steven Xu <xxu@student.unimelb.edu.au>
 #
-# URL: <http://nltk.org/>
+# URL: <https://www.nltk.org/>
 # For license information, see LICENSE.TXT
 
-import tempfile
 import os
+import tempfile
 import warnings
-from unittest import skip
 from subprocess import PIPE
 
 from nltk.internals import (
-    find_jar_iter,
-    config_java,
-    java,
     _java_options,
+    config_java,
+    find_jar_iter,
     find_jars_within_path,
+    java,
 )
-
 from nltk.parse.api import ParserI
 from nltk.parse.dependencygraph import DependencyGraph
 from nltk.tree import Tree
@@ -231,7 +228,7 @@ class GenericStanfordParser(ParserI):
         encoding = self._encoding
         cmd.extend(["-encoding", encoding])
         if self.corenlp_options:
-            cmd.append(self.corenlp_options)
+            cmd.extend(self.corenlp_options.split())
 
         default_options = " ".join(_java_options)
 
@@ -335,7 +332,7 @@ class StanfordParser(GenericStanfordParser):
             stacklevel=2,
         )
 
-        super(StanfordParser, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def _make_tree(self, result):
         return Tree.fromstring(result)
@@ -401,7 +398,7 @@ class StanfordDependencyParser(GenericStanfordParser):
             stacklevel=2,
         )
 
-        super(StanfordDependencyParser, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def _make_tree(self, result):
         return DependencyGraph(result, top_relation_label="root")
@@ -454,7 +451,7 @@ class StanfordNeuralDependencyParser(GenericStanfordParser):
             stacklevel=2,
         )
 
-        super(StanfordNeuralDependencyParser, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.corenlp_options += "-annotators tokenize,ssplit,pos,depparse"
 
     def tagged_parse_sents(self, sentences, verbose=False):
@@ -471,18 +468,3 @@ class StanfordNeuralDependencyParser(GenericStanfordParser):
 
     def _make_tree(self, result):
         return DependencyGraph(result, top_relation_label="ROOT")
-
-
-@skip("doctests from nltk.parse.stanford are skipped because it's deprecated")
-def setup_module(module):
-    from nose import SkipTest
-
-    try:
-        StanfordParser(
-            model_path="edu/stanford/nlp/models/lexparser/englishPCFG.ser.gz"
-        )
-        StanfordNeuralDependencyParser()
-    except LookupError:
-        raise SkipTest(
-            "doctests from nltk.parse.stanford are skipped because one of the stanford parser or CoreNLP jars doesn't exist"
-        )
