@@ -1,7 +1,6 @@
-# -*- coding: utf-8 -*-
 # Natural Language Toolkit: Probability and Statistics
 #
-# Copyright (C) 2001-2019 NLTK Project
+# Copyright (C) 2001-2022 NLTK Project
 # Author: Edward Loper <edloper@gmail.com>
 #         Steven Bird <stevenbird1@gmail.com> (additions)
 #         Trevor Cohn <tacohn@cs.mu.oz.au> (additions)
@@ -10,7 +9,7 @@
 #         Geoffrey Sampson <sampson@cantab.net> (additions)
 #         Ilia Kurenkov <ilia.kurenkov@gmail.com> (additions)
 #
-# URL: <http://nltk.org/>
+# URL: <https://www.nltk.org/>
 # For license information, see LICENSE.TXT
 
 """
@@ -37,29 +36,24 @@ implementation of the ``ConditionalProbDistI`` interface is
 ``ConditionalProbDist``, a derived distribution.
 
 """
-from __future__ import print_function, unicode_literals, division
 
+import array
 import math
 import random
 import warnings
-import array
-from collections import defaultdict, Counter
-from functools import reduce
 from abc import ABCMeta, abstractmethod
+from collections import Counter, defaultdict
+from functools import reduce
 
-from six import itervalues, text_type, add_metaclass
-
-from nltk import compat
 from nltk.internals import raise_unorderable_types
 
-_NINF = float('-1e300')
+_NINF = float("-1e300")
 
 ##//////////////////////////////////////////////////////
 ##  Frequency Distributions
 ##//////////////////////////////////////////////////////
 
 
-@compat.python_2_unicode_compatible
 class FreqDist(Counter):
     """
     A frequency distribution for the outcomes of an experiment.  A
@@ -129,28 +123,28 @@ class FreqDist(Counter):
         Override ``Counter.__setitem__()`` to invalidate the cached N
         """
         self._N = None
-        super(FreqDist, self).__setitem__(key, val)
+        super().__setitem__(key, val)
 
     def __delitem__(self, key):
         """
         Override ``Counter.__delitem__()`` to invalidate the cached N
         """
         self._N = None
-        super(FreqDist, self).__delitem__(key)
+        super().__delitem__(key)
 
     def update(self, *args, **kwargs):
         """
         Override ``Counter.update()`` to invalidate the cached N
         """
         self._N = None
-        super(FreqDist, self).update(*args, **kwargs)
+        super().update(*args, **kwargs)
 
     def setdefault(self, key, val):
         """
         Override ``Counter.setdefault()`` to invalidate the cached N
         """
         self._N = None
-        super(FreqDist, self).setdefault(key, val)
+        super().setdefault(key, val)
 
     def B(self):
         """
@@ -246,64 +240,71 @@ class FreqDist(Counter):
         """
         if len(self) == 0:
             raise ValueError(
-                'A FreqDist must have at least one sample before max is defined.'
+                "A FreqDist must have at least one sample before max is defined."
             )
         return self.most_common(1)[0][0]
 
-    def plot(self, *args, **kwargs):
+    def plot(
+        self, *args, title="", cumulative=False, percents=False, show=True, **kwargs
+    ):
         """
         Plot samples from the frequency distribution
         displaying the most frequent sample first.  If an integer
         parameter is supplied, stop after this many samples have been
-        plotted.  For a cumulative plot, specify cumulative=True.
+        plotted.  For a cumulative plot, specify cumulative=True. Additional
+        ``**kwargs`` are passed to matplotlib's plot function.
         (Requires Matplotlib to be installed.)
 
-        :param title: The title for the graph
+        :param title: The title for the graph.
         :type title: str
-        :param cumulative: A flag to specify whether the plot is cumulative (default = False)
-        :type title: bool
+        :param cumulative: Whether the plot is cumulative. (default = False)
+        :type cumulative: bool
+        :param percents: Whether the plot uses percents instead of counts. (default = False)
+        :type percents: bool
+        :param show: Whether to show the plot, or only return the ax.
+        :type show: bool
         """
         try:
             import matplotlib.pyplot as plt
-        except ImportError:
+        except ImportError as e:
             raise ValueError(
-                'The plot function requires matplotlib to be installed.'
-                'See http://matplotlib.org/'
-            )
+                "The plot function requires matplotlib to be installed."
+                "See https://matplotlib.org/"
+            ) from e
 
         if len(args) == 0:
             args = [len(self)]
         samples = [item for item, _ in self.most_common(*args)]
 
-        cumulative = _get_kwarg(kwargs, 'cumulative', False)
-        percents = _get_kwarg(kwargs, 'percents', False)
         if cumulative:
             freqs = list(self._cumulative_frequencies(samples))
-            ylabel = "Cumulative Counts"
-            if percents:
-                freqs = [f / freqs[len(freqs) - 1] * 100 for f in freqs]
-                ylabel = "Cumulative Percents"
+            ylabel = "Cumulative "
         else:
             freqs = [self[sample] for sample in samples]
-            ylabel = "Counts"
-        # percents = [f * 100 for f in freqs]  only in ProbDist?
+            ylabel = ""
+
+        if percents:
+            freqs = [f / self.N() * 100 for f in freqs]
+            ylabel += "Percents"
+        else:
+            ylabel += "Counts"
 
         ax = plt.gca()
-        ax.grid(True, color = "silver")
+        ax.grid(True, color="silver")
 
         if "linewidth" not in kwargs:
             kwargs["linewidth"] = 2
-        if "title" in kwargs:
-            ax.set_title(kwargs["title"])
-            del kwargs["title"]
+        if title:
+            ax.set_title(title)
 
         ax.plot(freqs, **kwargs)
         ax.set_xticks(range(len(samples)))
-        ax.set_xticklabels([text_type(s) for s in samples], rotation=90)
+        ax.set_xticklabels([str(s) for s in samples], rotation=90)
         ax.set_xlabel("Samples")
         ax.set_ylabel(ylabel)
 
-        plt.show()
+        if show:
+            plt.show()
 
         return ax
 
@@ -321,23 +322,25 @@ class FreqDist(Counter):
         """
         if len(args) == 0:
             args = [len(self)]
-        samples = [item for item, _ in self.most_common(*args)]
+        samples = _get_kwarg(
+            kwargs, "samples", [item for item, _ in self.most_common(*args)]
+        )
 
-        cumulative = _get_kwarg(kwargs, 'cumulative', False)
+        cumulative = _get_kwarg(kwargs, "cumulative", False)
         if cumulative:
             freqs = list(self._cumulative_frequencies(samples))
         else:
             freqs = [self[sample] for sample in samples]
         # percents = [f * 100 for f in freqs]  only in ProbDist?
 
-        width = max(len("%s" % s) for s in samples)
+        width = max(len(f"{s}") for s in samples)
         width = max(width, max(len("%d" % f) for f in freqs))
 
         for i in range(len(samples)):
-            print("%*s" % (width, samples[i]), end=' ')
+            print("%*s" % (width, samples[i]), end=" ")
         print()
         for i in range(len(samples)):
-            print("%*d" % (width, freqs[i]), end=' ')
+            print("%*d" % (width, freqs[i]), end=" ")
         print()
 
     def copy(self):
@@ -358,7 +361,7 @@ class FreqDist(Counter):
         FreqDist({'b': 4, 'c': 2, 'a': 1})
 
         """
-        return self.__class__(super(FreqDist, self).__add__(other))
+        return self.__class__(super().__add__(other))
 
     def __sub__(self, other):
         """
@@ -368,7 +371,7 @@ class FreqDist(Counter):
         FreqDist({'b': 2, 'a': 1})
 
         """
-        return self.__class__(super(FreqDist, self).__sub__(other))
+        return self.__class__(super().__sub__(other))
 
     def __or__(self, other):
         """
@@ -378,7 +381,7 @@ class FreqDist(Counter):
         FreqDist({'b': 3, 'c': 2, 'a': 1})
 
         """
-        return self.__class__(super(FreqDist, self).__or__(other))
+        return self.__class__(super().__or__(other))
 
     def __and__(self, other):
         """
@@ -388,19 +391,50 @@ class FreqDist(Counter):
         FreqDist({'b': 1})
 
         """
-        return self.__class__(super(FreqDist, self).__and__(other))
+        return self.__class__(super().__and__(other))
 
     def __le__(self, other):
+        """
+        Returns True if this frequency distribution is a subset of the other
+        and for no key the value exceeds the value of the same key from
+        the other frequency distribution.
+
+        The <= operator forms partial order and satisfying the axioms
+        reflexivity, antisymmetry and transitivity.
+
+        >>> FreqDist('a') <= FreqDist('a')
+        True
+        >>> a = FreqDist('abc')
+        >>> b = FreqDist('aabc')
+        >>> (a <= b, b <= a)
+        (True, False)
+        >>> FreqDist('a') <= FreqDist('abcd')
+        True
+        >>> FreqDist('abc') <= FreqDist('xyz')
+        False
+        >>> FreqDist('xyz') <= FreqDist('abc')
+        False
+        >>> c = FreqDist('a')
+        >>> d = FreqDist('aa')
+        >>> e = FreqDist('aaa')
+        >>> c <= d and d <= e and c <= e
+        True
+        """
         if not isinstance(other, FreqDist):
             raise_unorderable_types("<=", self, other)
         return set(self).issubset(other) and all(
             self[key] <= other[key] for key in self
         )
 
-    # @total_ordering doesn't work here, since the class inherits from a builtin class
-    __ge__ = lambda self, other: not self <= other or self == other
+    def __ge__(self, other):
+        if not isinstance(other, FreqDist):
+            raise_unorderable_types(">=", self, other)
+        return set(self).issuperset(other) and all(
+            self[key] >= other[key] for key in other
+        )
+
     __lt__ = lambda self, other: self <= other and not self == other
-    __gt__ = lambda self, other: not self <= other
+    __gt__ = lambda self, other: self >= other and not self == other
 
     def __repr__(self):
         """
@@ -428,10 +462,10 @@ class FreqDist(Counter):
         :type maxlen: int
         :rtype: string
         """
-        items = ['{0!r}: {1!r}'.format(*item) for item in self.most_common(maxlen)]
+        items = ["{!r}: {!r}".format(*item) for item in self.most_common(maxlen)]
         if len(self) > maxlen:
-            items.append('...')
-        return 'FreqDist({{{0}}})'.format(', '.join(items))
+            items.append("...")
+        return "FreqDist({{{0}}})".format(", ".join(items))
 
     def __str__(self):
         """
@@ -439,7 +473,16 @@ class FreqDist(Counter):
 
         :rtype: string
         """
-        return '<FreqDist with %d samples and %d outcomes>' % (len(self), self.N())
+        return "<FreqDist with %d samples and %d outcomes>" % (len(self), self.N())
+
+    def __iter__(self):
+        """
+        Return an iterator which yields tokens ordered by frequency.
+
+        :rtype: iterator
+        """
+        for token, _ in self.most_common(self.B()):
+            yield token
 
 
 ##//////////////////////////////////////////////////////
@@ -447,8 +490,7 @@ class FreqDist(Counter):
 ##//////////////////////////////////////////////////////
 
 
-@add_metaclass(ABCMeta)
-class ProbDistI(object):
+class ProbDistI(metaclass=ABCMeta):
     """
     A probability distribution for the outcomes of an experiment.  A
     probability distribution specifies how likely it is that an
@@ -551,7 +593,6 @@ class ProbDistI(object):
         return random.choice(list(self.samples()))
 
 
-@compat.python_2_unicode_compatible
 class UniformProbDist(ProbDistI):
     """
     A probability distribution that assigns equal probability to each
@@ -571,7 +612,7 @@ class UniformProbDist(ProbDistI):
         """
         if len(samples) == 0:
             raise ValueError(
-                'A Uniform probability distribution must ' + 'have at least one sample.'
+                "A Uniform probability distribution must " + "have at least one sample."
             )
         self._sampleset = set(samples)
         self._prob = 1.0 / len(self._sampleset)
@@ -587,10 +628,9 @@ class UniformProbDist(ProbDistI):
         return self._samples
 
     def __repr__(self):
-        return '<UniformProbDist with %d samples>' % len(self._sampleset)
+        return "<UniformProbDist with %d samples>" % len(self._sampleset)
 
 
-@compat.python_2_unicode_compatible
 class RandomProbDist(ProbDistI):
     """
     Generates a random probability distribution whereby each sample
@@ -601,7 +641,7 @@ class RandomProbDist(ProbDistI):
     def __init__(self, samples):
         if len(samples) == 0:
             raise ValueError(
-                'A probability distribution must ' + 'have at least one sample.'
+                "A probability distribution must " + "have at least one sample."
             )
         self._probs = self.unirand(samples)
         self._samples = list(self._probs.keys())
@@ -626,10 +666,10 @@ class RandomProbDist(ProbDistI):
             # can be subtracted from any element without risking probs not (0 1)
             randrow[-1] -= total - 1
 
-        return dict((s, randrow[i]) for i, s in enumerate(samples))
+        return {s: randrow[i] for i, s in enumerate(samples)}
 
     def max(self):
-        if not hasattr(self, '_max'):
+        if not hasattr(self, "_max"):
             self._max = max((p, v) for (v, p) in self._probs.items())[1]
         return self._max
 
@@ -640,10 +680,9 @@ class RandomProbDist(ProbDistI):
         return self._samples
 
     def __repr__(self):
-        return '<RandomUniformProbDist with %d samples>' % len(self._probs)
+        return "<RandomUniformProbDist with %d samples>" % len(self._probs)
 
 
-@compat.python_2_unicode_compatible
 class DictionaryProbDist(ProbDistI):
     """
     A probability distribution whose probabilities are directly
@@ -670,8 +709,8 @@ class DictionaryProbDist(ProbDistI):
         if normalize:
             if len(prob_dict) == 0:
                 raise ValueError(
-                    'A DictionaryProbDist must have at least one sample '
-                    + 'before it can be normalized.'
+                    "A DictionaryProbDist must have at least one sample "
+                    + "before it can be normalized."
                 )
             if log:
                 value_sum = sum_logs(list(self._prob_dict.values()))
@@ -711,7 +750,7 @@ class DictionaryProbDist(ProbDistI):
                 return math.log(self._prob_dict[sample], 2)
 
     def max(self):
-        if not hasattr(self, '_max'):
+        if not hasattr(self, "_max"):
             self._max = max((p, v) for (v, p) in self._prob_dict.items())[1]
         return self._max
 
@@ -719,10 +758,9 @@ class DictionaryProbDist(ProbDistI):
         return self._prob_dict.keys()
 
     def __repr__(self):
-        return '<ProbDist with %d samples>' % len(self._prob_dict)
+        return "<ProbDist with %d samples>" % len(self._prob_dict)
 
 
-@compat.python_2_unicode_compatible
 class MLEProbDist(ProbDistI):
     """
     The maximum likelihood estimate for the probability distribution
@@ -766,10 +804,9 @@ class MLEProbDist(ProbDistI):
         :rtype: str
         :return: A string representation of this ``ProbDist``.
         """
-        return '<MLEProbDist based on %d samples>' % self._freqdist.N()
+        return "<MLEProbDist based on %d samples>" % self._freqdist.N()
 
 
-@compat.python_2_unicode_compatible
 class LidstoneProbDist(ProbDistI):
     """
     The Lidstone estimate for the probability distribution of the
@@ -809,15 +846,15 @@ class LidstoneProbDist(ProbDistI):
         if (bins == 0) or (bins is None and freqdist.N() == 0):
             name = self.__class__.__name__[:-8]
             raise ValueError(
-                'A %s probability distribution ' % name + 'must have at least one bin.'
+                "A %s probability distribution " % name + "must have at least one bin."
             )
         if (bins is not None) and (bins < freqdist.B()):
             name = self.__class__.__name__[:-8]
             raise ValueError(
-                '\nThe number of bins in a %s distribution ' % name
-                + '(%d) must be greater than or equal to\n' % bins
-                + 'the number of bins in the FreqDist used '
-                + 'to create it (%d).' % freqdist.B()
+                "\nThe number of bins in a %s distribution " % name
+                + "(%d) must be greater than or equal to\n" % bins
+                + "the number of bins in the FreqDist used "
+                + "to create it (%d)." % freqdist.B()
             )
 
         self._freqdist = freqdist
@@ -867,10 +904,9 @@ class LidstoneProbDist(ProbDistI):
 
         :rtype: str
         """
-        return '<LidstoneProbDist based on %d samples>' % self._freqdist.N()
+        return "<LidstoneProbDist based on %d samples>" % self._freqdist.N()
 
 
-@compat.python_2_unicode_compatible
 class LaplaceProbDist(LidstoneProbDist):
     """
     The Laplace estimate for the probability distribution of the
@@ -904,10 +940,9 @@ class LaplaceProbDist(LidstoneProbDist):
         :rtype: str
         :return: A string representation of this ``ProbDist``.
         """
-        return '<LaplaceProbDist based on %d samples>' % self._freqdist.N()
+        return "<LaplaceProbDist based on %d samples>" % self._freqdist.N()
 
 
-@compat.python_2_unicode_compatible
 class ELEProbDist(LidstoneProbDist):
     """
     The expected likelihood estimate for the probability distribution
@@ -942,10 +977,9 @@ class ELEProbDist(LidstoneProbDist):
 
         :rtype: str
         """
-        return '<ELEProbDist based on %d samples>' % self._freqdist.N()
+        return "<ELEProbDist based on %d samples>" % self._freqdist.N()
 
 
-@compat.python_2_unicode_compatible
 class HeldoutProbDist(ProbDistI):
     """
     The heldout estimate for the probability distribution of the
@@ -1106,11 +1140,10 @@ class HeldoutProbDist(ProbDistI):
         :rtype: str
         :return: A string representation of this ``ProbDist``.
         """
-        s = '<HeldoutProbDist: %d base samples; %d heldout samples>'
+        s = "<HeldoutProbDist: %d base samples; %d heldout samples>"
         return s % (self._base_fdist.N(), self._heldout_fdist.N())
 
 
-@compat.python_2_unicode_compatible
 class CrossValidationProbDist(ProbDistI):
     """
     The cross-validation estimate for the probability distribution of
@@ -1159,7 +1192,7 @@ class CrossValidationProbDist(ProbDistI):
 
     def samples(self):
         # [xx] nb: this is not too efficient
-        return set(sum([list(fd) for fd in self._freqdists], []))
+        return set(sum((list(fd) for fd in self._freqdists), []))
 
     def prob(self, sample):
         # Find the average probability estimate returned by each
@@ -1178,10 +1211,9 @@ class CrossValidationProbDist(ProbDistI):
 
         :rtype: str
         """
-        return '<CrossValidationProbDist: %d-way>' % len(self._freqdists)
+        return "<CrossValidationProbDist: %d-way>" % len(self._freqdists)
 
 
-@compat.python_2_unicode_compatible
 class WittenBellProbDist(ProbDistI):
     """
     The Witten-Bell estimate of a probability distribution. This distribution
@@ -1225,7 +1257,7 @@ class WittenBellProbDist(ProbDistI):
         :type bins: int
         """
         assert bins is None or bins >= freqdist.B(), (
-            'bins parameter must not be less than %d=freqdist.B()' % freqdist.B()
+            "bins parameter must not be less than %d=freqdist.B()" % freqdist.B()
         )
         if bins is None:
             bins = freqdist.B()
@@ -1263,7 +1295,7 @@ class WittenBellProbDist(ProbDistI):
 
         :rtype: str
         """
-        return '<WittenBellProbDist based on %d samples>' % self._freqdist.N()
+        return "<WittenBellProbDist based on %d samples>" % self._freqdist.N()
 
 
 ##//////////////////////////////////////////////////////
@@ -1325,7 +1357,6 @@ class WittenBellProbDist(ProbDistI):
 ##//////////////////////////////////////////////////////
 
 
-@compat.python_2_unicode_compatible
 class SimpleGoodTuringProbDist(ProbDistI):
     """
     SimpleGoodTuring ProbDist approximates from frequency to frequency of
@@ -1336,7 +1367,7 @@ class SimpleGoodTuringProbDist(ProbDistI):
       Journal of Quantitative Linguistics, vol. 2 pp. 217-237.
     - "Speech and Language Processing (Jurafsky & Martin),
       2nd Edition, Chapter 4.5 p103 (log(Nc) =  a + b*log(c))
-    - http://www.grsampson.net/RGoodTur.html
+    - https://www.grsampson.net/RGoodTur.html
 
     Given a set of pair (xi, yi),  where the xi denotes the frequency and
     yi denotes the frequency of frequency, we want to minimize their
@@ -1360,7 +1391,7 @@ class SimpleGoodTuringProbDist(ProbDistI):
         """
         assert (
             bins is None or bins > freqdist.B()
-        ), 'bins parameter must not be less than %d=freqdist.B()+1' % (freqdist.B() + 1)
+        ), "bins parameter must not be less than %d=freqdist.B()+1" % (freqdist.B() + 1)
         if bins is None:
             bins = freqdist.B() + 1
         self._freqdist = freqdist
@@ -1419,10 +1450,10 @@ class SimpleGoodTuringProbDist(ProbDistI):
         self._slope = xy_cov / x_var if x_var != 0 else 0.0
         if self._slope >= -1:
             warnings.warn(
-                'SimpleGoodTuring did not find a proper best fit '
-                'line for smoothing probabilities of occurrences. '
-                'The probability estimates are likely to be '
-                'unreliable.'
+                "SimpleGoodTuring did not find a proper best fit "
+                "line for smoothing probabilities of occurrences. "
+                "The probability estimates are likely to be "
+                "unreliable."
             )
         self._intercept = y_mean - self._slope * x_mean
 
@@ -1546,7 +1577,7 @@ class SimpleGoodTuringProbDist(ProbDistI):
 
         :rtype: str
         """
-        return '<SimpleGoodTuringProbDist based on %d samples>' % self._freqdist.N()
+        return "<SimpleGoodTuringProbDist based on %d samples>" % self._freqdist.N()
 
 
 class MutableProbDist(ProbDistI):
@@ -1571,8 +1602,8 @@ class MutableProbDist(ProbDistI):
         :type store_logs: bool
         """
         self._samples = samples
-        self._sample_dict = dict((samples[i], i) for i in range(len(samples)))
-        self._data = array.array(str("d"), [0.0]) * len(samples)
+        self._sample_dict = {samples[i]: i for i in range(len(samples))}
+        self._data = array.array("d", [0.0]) * len(samples)
         for i in range(len(samples)):
             if store_logs:
                 self._data[i] = prob_dist.logprob(samples[i])
@@ -1599,7 +1630,7 @@ class MutableProbDist(ProbDistI):
         # inherit documentation
         i = self._sample_dict.get(sample)
         if i is None:
-            return float('-inf')
+            return float("-inf")
         return self._data[i] if self._logs else math.log(self._data[i], 2)
 
     def update(self, sample, prob, log=True):
@@ -1661,7 +1692,6 @@ class MutableProbDist(ProbDistI):
 # where possible.
 
 
-@compat.python_2_unicode_compatible
 class KneserNeyProbDist(ProbDistI):
     """
     Kneser-Ney estimate of a probability distribution. This is a version of
@@ -1710,7 +1740,7 @@ class KneserNeyProbDist(ProbDistI):
     def prob(self, trigram):
         # sample must be a triple
         if len(trigram) != 3:
-            raise ValueError('Expected an iterable with 3 members.')
+            raise ValueError("Expected an iterable with 3 members.")
         trigram = tuple(trigram)
         w0, w1, w2 = trigram
 
@@ -1768,12 +1798,12 @@ class KneserNeyProbDist(ProbDistI):
         return self._trigrams.max()
 
     def __repr__(self):
-        '''
+        """
         Return a string representation of this ProbDist
 
         :rtype: str
-        '''
-        return '<KneserNeyProbDist based on {0} trigrams'.format(self._trigrams.N())
+        """
+        return f"<KneserNeyProbDist based on {self._trigrams.N()} trigrams"
 
 
 ##//////////////////////////////////////////////////////
@@ -1783,7 +1813,7 @@ class KneserNeyProbDist(ProbDistI):
 
 def log_likelihood(test_pdist, actual_pdist):
     if not isinstance(test_pdist, ProbDistI) or not isinstance(actual_pdist, ProbDistI):
-        raise ValueError('expected a ProbDist.')
+        raise ValueError("expected a ProbDist.")
     # Is this right?
     return sum(
         actual_pdist.prob(s) * math.log(test_pdist.prob(s), 2) for s in actual_pdist
@@ -1800,7 +1830,6 @@ def entropy(pdist):
 ##//////////////////////////////////////////////////////
 
 
-@compat.python_2_unicode_compatible
 class ConditionalFreqDist(defaultdict):
     """
     A collection of frequency distributions for a single experiment
@@ -1888,66 +1917,97 @@ class ConditionalFreqDist(defaultdict):
 
         :rtype: int
         """
-        return sum(fdist.N() for fdist in itervalues(self))
+        return sum(fdist.N() for fdist in self.values())
 
-    def plot(self, *args, **kwargs):
+    def plot(
+        self,
+        *args,
+        samples=None,
+        title="",
+        cumulative=False,
+        percents=False,
+        conditions=None,
+        show=True,
+        **kwargs,
+    ):
         """
         Plot the given samples from the conditional frequency distribution.
-        For a cumulative plot, specify cumulative=True.
+        For a cumulative plot, specify cumulative=True. Additional ``*args`` and
+        ``**kwargs`` are passed to matplotlib's plot function.
         (Requires Matplotlib to be installed.)
 
         :param samples: The samples to plot
         :type samples: list
         :param title: The title for the graph
         :type title: str
+        :param cumulative: Whether the plot is cumulative. (default = False)
+        :type cumulative: bool
+        :param percents: Whether the plot uses percents instead of counts. (default = False)
+        :type percents: bool
         :param conditions: The conditions to plot (default is all)
         :type conditions: list
+        :param show: Whether to show the plot, or only return the ax.
+        :type show: bool
         """
         try:
-            from matplotlib import plt
-        except ImportError:
+            import matplotlib.pyplot as plt  # import statement fix
+        except ImportError as e:
             raise ValueError(
-                'The plot function requires matplotlib to be installed.'
-                'See http://matplotlib.org/'
-            )
+                "The plot function requires matplotlib to be installed."
+                "See https://matplotlib.org/"
+            ) from e
 
-        cumulative = _get_kwarg(kwargs, 'cumulative', False)
-        percents = _get_kwarg(kwargs, 'percents', False)
-        conditions = _get_kwarg(kwargs, 'conditions', sorted(self.conditions()))
-        title = _get_kwarg(kwargs, 'title', '')
-        samples = _get_kwarg(
-            kwargs, 'samples', sorted(set(v for c in conditions
-                                          if v in self
-                                          for v in self[c]))
-        )  # this computation could be wasted
+        if not conditions:
+            conditions = self.conditions()
+        else:
+            conditions = [c for c in conditions if c in self]
+        if not samples:
+            samples = sorted({v for c in conditions for v in self[c]})
         if "linewidth" not in kwargs:
             kwargs["linewidth"] = 2
+        ax = plt.gca()
+        if conditions:
+            freqs = []
+            for condition in conditions:
+                if cumulative:
+                    # freqs should be a list of list where each sub list will be a frequency of a condition
+                    freq = list(self[condition]._cumulative_frequencies(samples))
+                else:
+                    freq = [self[condition][sample] for sample in samples]
 
-        for condition in conditions:
-            if cumulative:
-                freqs = list(self[condition]._cumulative_frequencies(samples))
-                ylabel = "Cumulative Counts"
-                legend_loc = 'lower right'
                 if percents:
-                    freqs = [f / freqs[len(freqs) - 1] * 100 for f in freqs]
-                    ylabel = "Cumulative Percents"
+                    freq = [f / self[condition].N() * 100 for f in freq]
+
+                freqs.append(freq)
+
+            if cumulative:
+                ylabel = "Cumulative "
+                legend_loc = "lower right"
             else:
-                freqs = [self[condition][sample] for sample in samples]
-                ylabel = "Counts"
-                legend_loc = 'upper right'
-            # percents = [f * 100 for f in freqs] only in ConditionalProbDist?
-            kwargs['label'] = "%s" % condition
-            ax.plot(freqs, *args, **kwargs)
+                ylabel = ""
+                legend_loc = "upper right"
 
+            if percents:
+                ylabel += "Percents"
+            else:
+                ylabel += "Counts"
 
-        ax.legend(loc=legend_loc)
-        ax.grid(True, color="silver")
-        ax.set_xticks(range(len(samples)), [text_type(s) for s in samples], rotation=90)
-        if title:
-            ax.set_title(title)
-        ax.set_xlabel("Samples")
-        ax.set_ylabel(ylabel)
-        plt.show()
+            i = 0
+            for freq in freqs:
+                kwargs["label"] = conditions[i]  # label for each condition
+                i += 1
+                ax.plot(freq, *args, **kwargs)
+            ax.legend(loc=legend_loc)
+            ax.grid(True, color="silver")
+            ax.set_xticks(range(len(samples)))
+            ax.set_xticklabels([str(s) for s in samples], rotation=90)
+            if title:
+                ax.set_title(title)
+            ax.set_xlabel("Samples")
+            ax.set_ylabel(ylabel)
+
+        if show:
+            plt.show()
 
         return ax
 
@@ -1963,12 +2023,13 @@ class ConditionalFreqDist(defaultdict):
         :type title: bool
         """
 
-        cumulative = _get_kwarg(kwargs, 'cumulative', False)
-        conditions = _get_kwarg(kwargs, 'conditions', sorted(self.conditions()))
+        cumulative = _get_kwarg(kwargs, "cumulative", False)
+        conditions = _get_kwarg(kwargs, "conditions", sorted(self.conditions()))
         samples = _get_kwarg(
-            kwargs, 'samples', sorted(set(v for c in conditions
-                                          if c in self
-                                          for v in self[c])))  # this computation could be wasted
+            kwargs,
+            "samples",
+            sorted({v for c in conditions if c in self for v in self[c]}),
+        )  # this computation could be wasted
 
         width = max(len("%s" % s) for s in samples)
         freqs = dict()
@@ -1980,14 +2041,14 @@ class ConditionalFreqDist(defaultdict):
             width = max(width, max(len("%d" % f) for f in freqs[c]))
 
         condition_size = max(len("%s" % c) for c in conditions)
-        print(' ' * condition_size, end=' ')
+        print(" " * condition_size, end=" ")
         for s in samples:
-            print("%*s" % (width, s), end=' ')
+            print("%*s" % (width, s), end=" ")
         print()
         for c in conditions:
-            print("%*s" % (condition_size, c), end=' ')
+            print("%*s" % (condition_size, c), end=" ")
             for f in freqs[c]:
-                print("%*d" % (width, f), end=' ')
+                print("%*d" % (width, f), end=" ")
             print()
 
     # Mathematical operators
@@ -2088,12 +2149,10 @@ class ConditionalFreqDist(defaultdict):
 
         :rtype: str
         """
-        return '<ConditionalFreqDist with %d conditions>' % len(self)
+        return "<ConditionalFreqDist with %d conditions>" % len(self)
 
 
-@compat.python_2_unicode_compatible
-@add_metaclass(ABCMeta)
-class ConditionalProbDistI(dict):
+class ConditionalProbDistI(dict, metaclass=ABCMeta):
     """
     A collection of probability distributions for a single experiment
     run under different conditions.  Conditional probability
@@ -2129,7 +2188,7 @@ class ConditionalProbDistI(dict):
 
         :rtype: str
         """
-        return '<%s with %d conditions>' % (type(self).__name__, len(self))
+        return "<%s with %d conditions>" % (type(self).__name__, len(self))
 
 
 class ConditionalProbDist(ConditionalProbDistI):
@@ -2257,7 +2316,7 @@ def sum_logs(logs):
 ##//////////////////////////////////////////////////////
 
 
-class ProbabilisticMixIn(object):
+class ProbabilisticMixIn:
     """
     A mix-in class to associate probabilities with other classes
     (trees, rules, etc.).  To use the ``ProbabilisticMixIn`` class,
@@ -2295,13 +2354,13 @@ class ProbabilisticMixIn(object):
             the object.
         :type logprob: float
         """
-        if 'prob' in kwargs:
-            if 'logprob' in kwargs:
-                raise TypeError('Must specify either prob or logprob ' '(not both)')
+        if "prob" in kwargs:
+            if "logprob" in kwargs:
+                raise TypeError("Must specify either prob or logprob " "(not both)")
             else:
-                ProbabilisticMixIn.set_prob(self, kwargs['prob'])
-        elif 'logprob' in kwargs:
-            ProbabilisticMixIn.set_logprob(self, kwargs['logprob'])
+                ProbabilisticMixIn.set_prob(self, kwargs["prob"])
+        elif "logprob" in kwargs:
+            ProbabilisticMixIn.set_logprob(self, kwargs["logprob"])
         else:
             self.__prob = self.__logprob = None
 
@@ -2355,10 +2414,10 @@ class ProbabilisticMixIn(object):
 
 class ImmutableProbabilisticMixIn(ProbabilisticMixIn):
     def set_prob(self, prob):
-        raise ValueError('%s is immutable' % self.__class__.__name__)
+        raise ValueError("%s is immutable" % self.__class__.__name__)
 
     def set_logprob(self, prob):
-        raise ValueError('%s is immutable' % self.__class__.__name__)
+        raise ValueError("%s is immutable" % self.__class__.__name__)
 
 
 ## Helper function for processing keyword arguments
@@ -2451,83 +2510,81 @@ def demo(numsamples=6, numoutcomes=500):
 
     # Print the results in a formatted table.
     print(
-        (
-            '%d samples (1-%d); %d outcomes were sampled for each FreqDist'
-            % (numsamples, numsamples, numoutcomes)
-        )
+        "%d samples (1-%d); %d outcomes were sampled for each FreqDist"
+        % (numsamples, numsamples, numoutcomes)
     )
-    print('=' * 9 * (len(pdists) + 2))
-    FORMATSTR = '      FreqDist ' + '%8s ' * (len(pdists) - 1) + '|  Actual'
+    print("=" * 9 * (len(pdists) + 2))
+    FORMATSTR = "      FreqDist " + "%8s " * (len(pdists) - 1) + "|  Actual"
     print(FORMATSTR % tuple(repr(pdist)[1:9] for pdist in pdists[:-1]))
-    print('-' * 9 * (len(pdists) + 2))
-    FORMATSTR = '%3d   %8.6f ' + '%8.6f ' * (len(pdists) - 1) + '| %8.6f'
+    print("-" * 9 * (len(pdists) + 2))
+    FORMATSTR = "%3d   %8.6f " + "%8.6f " * (len(pdists) - 1) + "| %8.6f"
     for val in vals:
         print(FORMATSTR % val)
 
     # Print the totals for each column (should all be 1.0)
     zvals = list(zip(*vals))
     sums = [sum(val) for val in zvals[1:]]
-    print('-' * 9 * (len(pdists) + 2))
-    FORMATSTR = 'Total ' + '%8.6f ' * (len(pdists)) + '| %8.6f'
+    print("-" * 9 * (len(pdists) + 2))
+    FORMATSTR = "Total " + "%8.6f " * (len(pdists)) + "| %8.6f"
     print(FORMATSTR % tuple(sums))
-    print('=' * 9 * (len(pdists) + 2))
+    print("=" * 9 * (len(pdists) + 2))
 
     # Display the distributions themselves, if they're short enough.
     if len("%s" % fdist1) < 70:
-        print('  fdist1: %s' % fdist1)
-        print('  fdist2: %s' % fdist2)
-        print('  fdist3: %s' % fdist3)
+        print("  fdist1: %s" % fdist1)
+        print("  fdist2: %s" % fdist2)
+        print("  fdist3: %s" % fdist3)
     print()
 
-    print('Generating:')
+    print("Generating:")
     for pdist in pdists:
         fdist = FreqDist(pdist.generate() for i in range(5000))
-        print('%20s %s' % (pdist.__class__.__name__[:20], ("%s" % fdist)[:55]))
+        print("{:>20} {}".format(pdist.__class__.__name__[:20], ("%s" % fdist)[:55]))
     print()
 
 
 def gt_demo():
     from nltk import corpus
 
-    emma_words = corpus.gutenberg.words('austen-emma.txt')
+    emma_words = corpus.gutenberg.words("austen-emma.txt")
     fd = FreqDist(emma_words)
     sgt = SimpleGoodTuringProbDist(fd)
-    print('%18s %8s  %14s' % ("word", "freqency", "SimpleGoodTuring"))
+    print("{:>18} {:>8}  {:>14}".format("word", "frequency", "SimpleGoodTuring"))
     fd_keys_sorted = (
         key for key, value in sorted(fd.items(), key=lambda item: item[1], reverse=True)
     )
     for key in fd_keys_sorted:
-        print('%18s %8d  %14e' % (key, fd[key], sgt.prob(key)))
+        print("%18s %8d  %14e" % (key, fd[key], sgt.prob(key)))
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     demo(6, 10)
     demo(5, 5000)
     gt_demo()
 
 __all__ = [
-    'ConditionalFreqDist',
-    'ConditionalProbDist',
-    'ConditionalProbDistI',
-    'CrossValidationProbDist',
-    'DictionaryConditionalProbDist',
-    'DictionaryProbDist',
-    'ELEProbDist',
-    'FreqDist',
-    'SimpleGoodTuringProbDist',
-    'HeldoutProbDist',
-    'ImmutableProbabilisticMixIn',
-    'LaplaceProbDist',
-    'LidstoneProbDist',
-    'MLEProbDist',
-    'MutableProbDist',
-    'KneserNeyProbDist',
-    'ProbDistI',
-    'ProbabilisticMixIn',
-    'UniformProbDist',
-    'WittenBellProbDist',
-    'add_logs',
-    'log_likelihood',
-    'sum_logs',
-    'entropy',
+    "ConditionalFreqDist",
+    "ConditionalProbDist",
+    "ConditionalProbDistI",
+    "CrossValidationProbDist",
+    "DictionaryConditionalProbDist",
+    "DictionaryProbDist",
+    "ELEProbDist",
+    "FreqDist",
+    "SimpleGoodTuringProbDist",
+    "HeldoutProbDist",
+    "ImmutableProbabilisticMixIn",
+    "LaplaceProbDist",
+    "LidstoneProbDist",
+    "MLEProbDist",
+    "MutableProbDist",
+    "KneserNeyProbDist",
+    "ProbDistI",
+    "ProbabilisticMixIn",
+    "UniformProbDist",
+    "WittenBellProbDist",
+    "add_logs",
+    "log_likelihood",
+    "sum_logs",
+    "entropy",
 ]
