@@ -2,8 +2,8 @@
 #
 # Author: Dan Garrette <dhgarrette@gmail.com>
 #
-# Copyright (C) 2001-2020 NLTK Project
-# URL: <http://nltk.org/>
+# Copyright (C) 2001-2022 NLTK Project
+# URL: <https://www.nltk.org/>
 # For license information, see LICENSE.TXT
 
 import operator
@@ -27,10 +27,10 @@ from nltk.sem.logic import (
     ImpExpression,
     IndividualVariableExpression,
     LambdaExpression,
-    Tokens,
     LogicParser,
     NegatedExpression,
     OrExpression,
+    Tokens,
     Variable,
     is_eventvar,
     is_funcvar,
@@ -42,6 +42,7 @@ from nltk.sem.logic import (
 try:
     from tkinter import Canvas, Tk
     from tkinter.font import Font
+
     from nltk.util import in_idle
 
 except ImportError:
@@ -194,7 +195,7 @@ class DrtParser(LogicParser):
         return DrtLambdaExpression(variables, term)
 
 
-class DrtExpression(object):
+class DrtExpression:
     """
     This is the base abstract DRT Expression from which every DRT
     Expression extends.
@@ -263,7 +264,7 @@ class DrtExpression(object):
         raise NotImplementedError()
 
     def is_pronoun_function(self):
-        """ Is self of the form "PRO(x)"? """
+        """Is self of the form "PRO(x)"?"""
         return (
             isinstance(self, DrtApplicationExpression)
             and isinstance(self.function, DrtAbstractVariableExpression)
@@ -303,7 +304,7 @@ class DRS(DrtExpression, Expression):
     def __init__(self, refs, conds, consequent=None):
         """
         :param refs: list of ``DrtIndividualVariableExpression`` for the
-        discourse referents
+            discourse referents
         :param conds: list of ``Expression`` for the conditions
         """
         self.refs = refs
@@ -383,7 +384,7 @@ class DRS(DrtExpression, Expression):
         """:see: AbstractExpression.get_refs()"""
         if recursive:
             conds_refs = self.refs + list(
-                chain(*(c.get_refs(True) for c in self.conds))
+                chain.from_iterable(c.get_refs(True) for c in self.conds)
             )
             if self.consequent:
                 conds_refs.extend(self.consequent.get_refs(True))
@@ -414,7 +415,7 @@ class DRS(DrtExpression, Expression):
                 and isinstance(cond.second, AbstractVariableExpression)
             ):
                 drs = DRS(
-                    list(set(drs.refs) - set([cond.second.variable])),
+                    list(set(drs.refs) - {cond.second.variable}),
                     drs.conds[:i] + drs.conds[i + 1 :],
                     drs.consequent,
                 )
@@ -535,7 +536,7 @@ class DRS(DrtExpression, Expression):
     __hash__ = Expression.__hash__
 
     def __str__(self):
-        drs = "([%s],[%s])" % (
+        drs = "([{}],[{}])".format(
             ",".join(self._order_ref_strings(self.refs)),
             ", ".join("%s" % cond for cond in self.conds),
         )  # map(str, self.conds)))
@@ -665,7 +666,7 @@ class DrtProposition(DrtExpression, Expression):
         return combinator(self.variable, function(self.drs))
 
     def __str__(self):
-        return "prop(%s, %s)" % (self.variable, self.drs)
+        return f"prop({self.variable}, {self.drs})"
 
 
 class DrtNegatedExpression(DrtExpression, NegatedExpression):
@@ -711,11 +712,17 @@ class DrtLambdaExpression(DrtExpression, LambdaExpression):
         blank = " " * len(var_string)
         return (
             ["    " + blank + line for line in term_lines[:1]]
-            + [" \  " + blank + line for line in term_lines[1:2]]
-            + [" /\ " + var_string + line for line in term_lines[2:3]]
+            + [r" \  " + blank + line for line in term_lines[1:2]]
+            + [r" /\ " + var_string + line for line in term_lines[2:3]]
             + ["    " + blank + line for line in term_lines[3:]]
         )
 
+    def get_refs(self, recursive=False):
+        """:see: AbstractExpression.get_refs()"""
+        return (
+            [self.variable] + self.term.get_refs(True) if recursive else [self.variable]
+        )
+
 
 class DrtBinaryExpression(DrtExpression, BinaryExpression):
     def get_refs(self, recursive=False):
@@ -891,7 +898,7 @@ class DrtConcatenation(DrtBooleanExpression):
         )
         if self.consequent:
             drs = DrtBinaryExpression._assemble_pretty(
-                drs, DrtTokens.IMP, self._pretty(self.consequent)
+                drs, DrtTokens.IMP, self.consequent._pretty()
             )
         return drs
 
@@ -1085,10 +1092,10 @@ def resolve_anaphora(expression, trail=[]):
         )
 
 
-class DrsDrawer(object):
+class DrsDrawer:
     BUFFER = 3  # Space between elements
     TOPSPACE = 10  # Space above whole DRS
-    OUTERSPACE = 6  # Space to the left, right, and bottom of the whle DRS
+    OUTERSPACE = 6  # Space to the left, right, and bottom of the while DRS
 
     def __init__(self, drs, size_canvas=True, canvas=None):
         """
@@ -1427,10 +1434,8 @@ def demo():
 def test_draw():
     try:
         from tkinter import Tk
-    except ImportError:
-        from nose import SkipTest
-
-        raise SkipTest("tkinter is required, but it's not available.")
+    except ImportError as e:
+        raise ValueError("tkinter is required, but it's not available.")
 
     expressions = [
         r"x",
