@@ -2,63 +2,59 @@
 #
 # Author: Dan Garrette <dhgarrette@gmail.com>
 #
-# Copyright (C) 2001-2019 NLTK Project
-# URL: <http://nltk.org>
+# Copyright (C) 2001-2022 NLTK Project
+# URL: <https://www.nltk.org/>
 # For license information, see LICENSE.TXT
 
 """
 A version of first order predicate logic, built on
 top of the typed lambda calculus.
 """
-from __future__ import print_function, unicode_literals
 
-import re
 import operator
+import re
 from collections import defaultdict
 from functools import reduce, total_ordering
 
-from six import string_types
-
-from nltk.util import Trie
 from nltk.internals import Counter
-from nltk.compat import python_2_unicode_compatible
+from nltk.util import Trie
 
-APP = 'APP'
+APP = "APP"
 
 _counter = Counter()
 
 
-class Tokens(object):
-    LAMBDA = '\\'
-    LAMBDA_LIST = ['\\']
+class Tokens:
+    LAMBDA = "\\"
+    LAMBDA_LIST = ["\\"]
 
     # Quantifiers
-    EXISTS = 'exists'
-    EXISTS_LIST = ['some', 'exists', 'exist']
-    ALL = 'all'
-    ALL_LIST = ['all', 'forall']
+    EXISTS = "exists"
+    EXISTS_LIST = ["some", "exists", "exist"]
+    ALL = "all"
+    ALL_LIST = ["all", "forall"]
 
     # Punctuation
-    DOT = '.'
-    OPEN = '('
-    CLOSE = ')'
-    COMMA = ','
+    DOT = "."
+    OPEN = "("
+    CLOSE = ")"
+    COMMA = ","
 
     # Operations
-    NOT = '-'
-    NOT_LIST = ['not', '-', '!']
-    AND = '&'
-    AND_LIST = ['and', '&', '^']
-    OR = '|'
-    OR_LIST = ['or', '|']
-    IMP = '->'
-    IMP_LIST = ['implies', '->', '=>']
-    IFF = '<->'
-    IFF_LIST = ['iff', '<->', '<=>']
-    EQ = '='
-    EQ_LIST = ['=', '==']
-    NEQ = '!='
-    NEQ_LIST = ['!=']
+    NOT = "-"
+    NOT_LIST = ["not", "-", "!"]
+    AND = "&"
+    AND_LIST = ["and", "&", "^"]
+    OR = "|"
+    OR_LIST = ["or", "|"]
+    IMP = "->"
+    IMP_LIST = ["implies", "->", "=>"]
+    IFF = "<->"
+    IFF_LIST = ["iff", "<->", "<=>"]
+    EQ = "="
+    EQ_LIST = ["=", "=="]
+    NEQ = "!="
+    NEQ_LIST = ["!="]
 
     # Collections of tokens
     BINOPS = AND_LIST + OR_LIST + IMP_LIST + IFF_LIST
@@ -68,7 +64,7 @@ class Tokens(object):
     TOKENS = BINOPS + EQ_LIST + NEQ_LIST + QUANTS + LAMBDA_LIST + PUNCT + NOT_LIST
 
     # Special
-    SYMBOLS = [x for x in TOKENS if re.match(r'^[-\\.(),!&^|>=<]*$', x)]
+    SYMBOLS = [x for x in TOKENS if re.match(r"^[-\\.(),!&^|>=<]*$", x)]
 
 
 def boolean_ops():
@@ -98,14 +94,14 @@ def binding_ops():
         print("%-15s\t%s" % pair)
 
 
-@python_2_unicode_compatible
-class LogicParser(object):
+class LogicParser:
     """A lambda calculus expression parser."""
 
     def __init__(self, type_check=False):
         """
-        :param type_check: bool should type checking be performed?
-        to their types.
+        :param type_check: should type checking be performed
+            to their types?
+        :type type_check: bool
         """
         assert isinstance(type_check, bool)
 
@@ -144,7 +140,7 @@ class LogicParser(object):
 
         :param data: str for the input to be parsed
         :param signature: ``dict<str, str>`` that maps variable names to type
-        strings
+            strings
         :returns: a parsed Expression
         """
         data = data.rstrip()
@@ -157,8 +153,8 @@ class LogicParser(object):
             if self.inRange(0):
                 raise UnexpectedTokenException(self._currentIndex + 1, self.token(0))
         except LogicalExpressionException as e:
-            msg = '%s\n%s\n%s^' % (e, data, ' ' * mapping[e.index - 1])
-            raise LogicalExpressionException(None, msg)
+            msg = "{}\n{}\n{}^".format(e, data, " " * mapping[e.index - 1])
+            raise LogicalExpressionException(None, msg) from e
 
         if self.type_check:
             result.typecheck(signature)
@@ -170,7 +166,7 @@ class LogicParser(object):
         out = []
         mapping = {}
         tokenTrie = Trie(self.get_all_symbols())
-        token = ''
+        token = ""
         data_idx = 0
         token_start_idx = data_idx
         while data_idx < len(data):
@@ -184,7 +180,7 @@ class LogicParser(object):
 
             st = tokenTrie
             c = data[data_idx]
-            symbol = ''
+            symbol = ""
             while c in st:
                 symbol += c
                 st = st[c]
@@ -197,16 +193,16 @@ class LogicParser(object):
                 if token:
                     mapping[len(out)] = token_start_idx
                     out.append(token)
-                    token = ''
+                    token = ""
                 mapping[len(out)] = data_idx
                 out.append(symbol)
                 data_idx += len(symbol)
             else:
-                if data[data_idx] in ' \t\n':  # any whitespace
+                if data[data_idx] in " \t\n":  # any whitespace
                     if token:
                         mapping[len(out)] = token_start_idx
                         out.append(token)
-                        token = ''
+                        token = ""
                 else:
                     if not token:
                         token_start_idx = data_idx
@@ -220,7 +216,7 @@ class LogicParser(object):
         return out, mapping
 
     def process_quoted_token(self, data_idx, data):
-        token = ''
+        token = ""
         c = data[data_idx]
         i = data_idx
         for start, end, escape, incl_quotes in self.quote_chars:
@@ -251,7 +247,7 @@ class LogicParser(object):
                     token += data[i]
                 i += 1
                 if not token:
-                    raise LogicalExpressionException(None, 'Empty quoted token found')
+                    raise LogicalExpressionException(None, "Empty quoted token found")
                 break
         return token, i
 
@@ -274,8 +270,8 @@ class LogicParser(object):
             else:
                 tok = self._buffer[self._currentIndex + location]
             return tok
-        except IndexError:
-            raise ExpectedMoreTokensException(self._currentIndex + 1)
+        except IndexError as e:
+            raise ExpectedMoreTokensException(self._currentIndex + 1) from e
 
     def isvariable(self, tok):
         return tok not in Tokens.TOKENS
@@ -284,16 +280,16 @@ class LogicParser(object):
         """Parse the next complete expression from the stream and return it."""
         try:
             tok = self.token()
-        except ExpectedMoreTokensException:
+        except ExpectedMoreTokensException as e:
             raise ExpectedMoreTokensException(
-                self._currentIndex + 1, message='Expression expected.'
-            )
+                self._currentIndex + 1, message="Expression expected."
+            ) from e
 
         accum = self.handle(tok, context)
 
         if not accum:
             raise UnexpectedTokenException(
-                self._currentIndex, tok, message='Expression expected.'
+                self._currentIndex, tok, message="Expression expected."
             )
 
         return self.attempt_adjuncts(accum, context)
@@ -365,7 +361,7 @@ class LogicParser(object):
         try:
             tok = self.token()
         except ExpectedMoreTokensException as e:
-            raise ExpectedMoreTokensException(e.index, 'Variable expected.')
+            raise ExpectedMoreTokensException(e.index, "Variable expected.") from e
         if isinstance(self.make_VariableExpression(tok), ConstantExpression):
             raise LogicalExpressionException(
                 self._currentIndex,
@@ -381,7 +377,7 @@ class LogicParser(object):
                 self._currentIndex + 2,
                 message="Variable and Expression expected following lambda operator.",
             )
-        vars = [self.get_next_token_variable('abstracted')]
+        vars = [self.get_next_token_variable("abstracted")]
         while True:
             if not self.inRange(0) or (
                 self.token(0) == Tokens.DOT and not self.inRange(1)
@@ -392,7 +388,7 @@ class LogicParser(object):
             if not self.isvariable(self.token(0)):
                 break
             # Support expressions like: \x y.M == \x.\y.M
-            vars.append(self.get_next_token_variable('abstracted'))
+            vars.append(self.get_next_token_variable("abstracted"))
         if self.inRange(0) and self.token(0) == Tokens.DOT:
             self.token()  # swallow the dot
 
@@ -411,7 +407,7 @@ class LogicParser(object):
                 message="Variable and Expression expected following quantifier '%s'."
                 % tok,
             )
-        vars = [self.get_next_token_variable('quantified')]
+        vars = [self.get_next_token_variable("quantified")]
         while True:
             if not self.inRange(0) or (
                 self.token(0) == Tokens.DOT and not self.inRange(1)
@@ -422,7 +418,7 @@ class LogicParser(object):
             if not self.isvariable(self.token(0)):
                 break
             # Support expressions like: some x y.M == some x.some y.M
-            vars.append(self.get_next_token_variable('quantified'))
+            vars.append(self.get_next_token_variable("quantified"))
         if self.inRange(0) and self.token(0) == Tokens.DOT:
             self.token()  # swallow the dot
 
@@ -563,7 +559,7 @@ class LogicParser(object):
         except ExpectedMoreTokensException as e:
             raise ExpectedMoreTokensException(
                 e.index, message="Expected token '%s'." % expected
-            )
+            ) from e
 
         if isinstance(expected, list):
             if tok not in expected:
@@ -582,10 +578,10 @@ class LogicParser(object):
 
     def __repr__(self):
         if self.inRange(0):
-            msg = 'Next token: ' + self.token(0)
+            msg = "Next token: " + self.token(0)
         else:
-            msg = 'No more tokens'
-        return '<' + self.__class__.__name__ + ': ' + msg + '>'
+            msg = "No more tokens"
+        return "<" + self.__class__.__name__ + ": " + msg + ">"
 
 
 def read_logic(s, logic_parser=None, encoding=None):
@@ -609,23 +605,22 @@ def read_logic(s, logic_parser=None, encoding=None):
     statements = []
     for linenum, line in enumerate(s.splitlines()):
         line = line.strip()
-        if line.startswith('#') or line == '':
+        if line.startswith("#") or line == "":
             continue
         try:
             statements.append(logic_parser.parse(line))
-        except LogicalExpressionException:
-            raise ValueError('Unable to parse line %s: %s' % (linenum, line))
+        except LogicalExpressionException as e:
+            raise ValueError(f"Unable to parse line {linenum}: {line}") from e
     return statements
 
 
 @total_ordering
-@python_2_unicode_compatible
-class Variable(object):
+class Variable:
     def __init__(self, name):
         """
         :param name: the name of the variable
         """
-        assert isinstance(name, string_types), "%s is not a string" % name
+        assert isinstance(name, str), "%s is not a string" % name
         self.name = name
 
     def __eq__(self, other):
@@ -664,19 +659,19 @@ def unique_variable(pattern=None, ignore=None):
     """
     if pattern is not None:
         if is_indvar(pattern.name):
-            prefix = 'z'
+            prefix = "z"
         elif is_funcvar(pattern.name):
-            prefix = 'F'
+            prefix = "F"
         elif is_eventvar(pattern.name):
-            prefix = 'e0'
+            prefix = "e0"
         else:
             assert False, "Cannot generate a unique constant"
     else:
-        prefix = 'z'
+        prefix = "z"
 
-    v = Variable("%s%s" % (prefix, _counter.get()))
+    v = Variable(f"{prefix}{_counter.get()}")
     while ignore is not None and v in ignore:
-        v = Variable("%s%s" % (prefix, _counter.get()))
+        v = Variable(f"{prefix}{_counter.get()}")
     return v
 
 
@@ -685,15 +680,14 @@ def skolem_function(univ_scope=None):
     Return a skolem function over the variables in univ_scope
     param univ_scope
     """
-    skolem = VariableExpression(Variable('F%s' % _counter.get()))
+    skolem = VariableExpression(Variable("F%s" % _counter.get()))
     if univ_scope:
         for v in list(univ_scope):
             skolem = skolem(VariableExpression(v))
     return skolem
 
 
-@python_2_unicode_compatible
-class Type(object):
+class Type:
     def __repr__(self):
         return "%s" % self
 
@@ -705,7 +699,6 @@ class Type(object):
         return read_type(s)
 
 
-@python_2_unicode_compatible
 class ComplexType(Type):
     def __init__(self, first, second):
         assert isinstance(first, Type), "%s is not a Type" % first
@@ -750,13 +743,13 @@ class ComplexType(Type):
         if self == ANY_TYPE:
             return "%s" % ANY_TYPE
         else:
-            return '<%s,%s>' % (self.first, self.second)
+            return f"<{self.first},{self.second}>"
 
     def str(self):
         if self == ANY_TYPE:
             return ANY_TYPE.str()
         else:
-            return '(%s -> %s)' % (self.first.str(), self.second.str())
+            return f"({self.first.str()} -> {self.second.str()})"
 
 
 class BasicType(Type):
@@ -778,34 +771,30 @@ class BasicType(Type):
             return None
 
 
-@python_2_unicode_compatible
 class EntityType(BasicType):
     def __str__(self):
-        return 'e'
+        return "e"
 
     def str(self):
-        return 'IND'
+        return "IND"
 
 
-@python_2_unicode_compatible
 class TruthValueType(BasicType):
     def __str__(self):
-        return 't'
+        return "t"
 
     def str(self):
-        return 'BOOL'
+        return "BOOL"
 
 
-@python_2_unicode_compatible
 class EventType(BasicType):
     def __str__(self):
-        return 'v'
+        return "v"
 
     def str(self):
-        return 'EVENT'
+        return "EVENT"
 
 
-@python_2_unicode_compatible
 class AnyType(BasicType, ComplexType):
     def __init__(self):
         pass
@@ -833,10 +822,10 @@ class AnyType(BasicType, ComplexType):
         return other
 
     def __str__(self):
-        return '?'
+        return "?"
 
     def str(self):
-        return 'ANY'
+        return "ANY"
 
 
 TRUTH_TYPE = TruthValueType()
@@ -846,19 +835,19 @@ ANY_TYPE = AnyType()
 
 
 def read_type(type_string):
-    assert isinstance(type_string, string_types)
-    type_string = type_string.replace(' ', '')  # remove spaces
+    assert isinstance(type_string, str)
+    type_string = type_string.replace(" ", "")  # remove spaces
 
-    if type_string[0] == '<':
-        assert type_string[-1] == '>'
+    if type_string[0] == "<":
+        assert type_string[-1] == ">"
         paren_count = 0
         for i, char in enumerate(type_string):
-            if char == '<':
+            if char == "<":
                 paren_count += 1
-            elif char == '>':
+            elif char == ">":
                 paren_count -= 1
                 assert paren_count > 0
-            elif char == ',':
+            elif char == ",":
                 if paren_count == 1:
                     break
         return ComplexType(
@@ -871,12 +860,14 @@ def read_type(type_string):
     elif type_string[0] == "%s" % ANY_TYPE:
         return ANY_TYPE
     else:
-        raise LogicalExpressionException(None, "Unexpected character: '%s'." % type_string[0])
+        raise LogicalExpressionException(
+            None, "Unexpected character: '%s'." % type_string[0]
+        )
 
 
 class TypeException(Exception):
     def __init__(self, msg):
-        super(TypeException, self).__init__(msg)
+        super().__init__(msg)
 
 
 class InconsistentTypeHierarchyException(TypeException):
@@ -891,12 +882,12 @@ class InconsistentTypeHierarchyException(TypeException):
                 "The variable '%s' was found in multiple places with different"
                 " types." % (variable)
             )
-        super(InconsistentTypeHierarchyException, self).__init__(msg)
+        super().__init__(msg)
 
 
 class TypeResolutionException(TypeException):
     def __init__(self, expression, other_type):
-        super(TypeResolutionException, self).__init__(
+        super().__init__(
             "The type of '%s', '%s', cannot be resolved with type '%s'"
             % (expression, expression.type, other_type)
         )
@@ -904,7 +895,7 @@ class TypeResolutionException(TypeException):
 
 class IllegalTypeException(TypeException):
     def __init__(self, expression, other_type, allowed_type):
-        super(IllegalTypeException, self).__init__(
+        super().__init__(
             "Cannot set type of %s '%s' to '%s'; must match type '%s'."
             % (expression.__class__.__name__, expression, other_type, allowed_type)
         )
@@ -926,7 +917,7 @@ def typecheck(expressions, signature=None):
     return signature
 
 
-class SubstituteBindingsI(object):
+class SubstituteBindingsI:
     """
     An interface for classes that can perform substitutions for
     variables.
@@ -948,7 +939,6 @@ class SubstituteBindingsI(object):
         raise NotImplementedError()
 
 
-@python_2_unicode_compatible
 class Expression(SubstituteBindingsI):
     """This is the base abstract object for all logical expressions"""
 
@@ -1036,8 +1026,8 @@ class Expression(SubstituteBindingsI):
                     val = self.make_VariableExpression(val)
                 elif not isinstance(val, Expression):
                     raise ValueError(
-                        'Can not substitute a non-expression '
-                        'value into an expression: %r' % (val,)
+                        "Can not substitute a non-expression "
+                        "value into an expression: %r" % (val,)
                     )
                 # Substitute bindings in the target value.
                 val = val.substitute_bindings(bindings)
@@ -1066,7 +1056,7 @@ class Expression(SubstituteBindingsI):
 
         self._set_type(signature=sig)
 
-        return dict((key, sig[key][0].type) for key in sig)
+        return {key: sig[key][0].type for key in sig}
 
     def findtype(self, variable):
         """
@@ -1110,7 +1100,7 @@ class Expression(SubstituteBindingsI):
 
         def get_indiv_vars(e):
             if isinstance(e, IndividualVariableExpression):
-                return set([e])
+                return {e}
             elif isinstance(e, AbstractVariableExpression):
                 return set()
             else:
@@ -1121,9 +1111,9 @@ class Expression(SubstituteBindingsI):
         result = self
         for i, e in enumerate(sorted(get_indiv_vars(self), key=lambda e: e.variable)):
             if isinstance(e, EventVariableExpression):
-                newVar = e.__class__(Variable('e0%s' % (i + 1)))
+                newVar = e.__class__(Variable("e0%s" % (i + 1)))
             elif isinstance(e, IndividualVariableExpression):
-                newVar = e.__class__(Variable('z%s' % (i + 1)))
+                newVar = e.__class__(Variable("z%s" % (i + 1)))
             else:
                 newVar = e
             result = result.replace(e.variable, newVar, True)
@@ -1162,7 +1152,7 @@ class Expression(SubstituteBindingsI):
         return self.visit(function, lambda parts: combinator(*parts))
 
     def __repr__(self):
-        return '<%s %s>' % (self.__class__.__name__, self)
+        return f"<{self.__class__.__name__} {self}>"
 
     def __str__(self):
         return self.str()
@@ -1174,9 +1164,9 @@ class Expression(SubstituteBindingsI):
         variables and any variable starting with '?' or '@'.
         :return: set of ``Variable`` objects
         """
-        return self.free() | set(
-            p for p in self.predicates() | self.constants() if re.match('^[?@]', p.name)
-        )
+        return self.free() | {
+            p for p in self.predicates() | self.constants() if re.match("^[?@]", p.name)
+        }
 
     def free(self):
         """
@@ -1216,7 +1206,6 @@ class Expression(SubstituteBindingsI):
         return VariableExpression(variable)
 
 
-@python_2_unicode_compatible
 class ApplicationExpression(Expression):
     r"""
     This class is used to represent two related types of logical expressions.
@@ -1283,7 +1272,7 @@ class ApplicationExpression(Expression):
             self.function._set_type(
                 ComplexType(self.argument.type, other_type), signature
             )
-        except TypeResolutionException:
+        except TypeResolutionException as e:
             raise TypeException(
                 "The function '%s' is of type '%s' and cannot be applied "
                 "to '%s' of type '%s'.  Its argument must match type '%s'."
@@ -1294,7 +1283,7 @@ class ApplicationExpression(Expression):
                     self.argument.type,
                     self.function.type.first,
                 )
-            )
+            ) from e
 
     def findtype(self, variable):
         """:see Expression.findtype()"""
@@ -1334,7 +1323,7 @@ class ApplicationExpression(Expression):
     def predicates(self):
         """:see: Expression.predicates()"""
         if isinstance(self.function, ConstantExpression):
-            function_preds = set([self.function.variable])
+            function_preds = {self.function.variable}
         else:
             function_preds = self.function.predicates()
         return function_preds | self.argument.predicates()
@@ -1359,7 +1348,7 @@ class ApplicationExpression(Expression):
         # uncurry the arguments and find the base function
         if self.is_atom():
             function, args = self.uncurry()
-            arg_str = ','.join("%s" % arg for arg in args)
+            arg_str = ",".join("%s" % arg for arg in args)
         else:
             # Leave arguments curried
             function = self.function
@@ -1420,7 +1409,6 @@ class ApplicationExpression(Expression):
 
 
 @total_ordering
-@python_2_unicode_compatible
 class AbstractVariableExpression(Expression):
     """This class represents a variable to be used as a predicate or entity"""
 
@@ -1519,7 +1507,7 @@ class IndividualVariableExpression(AbstractVariableExpression):
 
     def free(self):
         """:see: Expression.free()"""
-        return set([self.variable])
+        return {self.variable}
 
     def constants(self):
         """:see: Expression.constants()"""
@@ -1534,7 +1522,7 @@ class FunctionVariableExpression(AbstractVariableExpression):
 
     def free(self):
         """:see: Expression.free()"""
-        return set([self.variable])
+        return {self.variable}
 
     def constants(self):
         """:see: Expression.constants()"""
@@ -1584,7 +1572,7 @@ class ConstantExpression(AbstractVariableExpression):
 
     def constants(self):
         """:see: Expression.constants()"""
-        return set([self.variable])
+        return {self.variable}
 
 
 def VariableExpression(variable):
@@ -1659,7 +1647,7 @@ class VariableBinderExpression(Expression):
 
     def free(self):
         """:see: Expression.free()"""
-        return self.term.free() - set([self.variable])
+        return self.term.free() - {self.variable}
 
     def findtype(self, variable):
         """:see Expression.findtype()"""
@@ -1696,7 +1684,6 @@ class VariableBinderExpression(Expression):
     __hash__ = Expression.__hash__
 
 
-@python_2_unicode_compatible
 class LambdaExpression(VariableBinderExpression):
     @property
     def type(self):
@@ -1721,13 +1708,12 @@ class LambdaExpression(VariableBinderExpression):
             term = term.term
         return (
             Tokens.LAMBDA
-            + ' '.join("%s" % v for v in variables)
+            + " ".join("%s" % v for v in variables)
             + Tokens.DOT
             + "%s" % term
         )
 
 
-@python_2_unicode_compatible
 class QuantifiedExpression(VariableBinderExpression):
     @property
     def type(self):
@@ -1752,8 +1738,8 @@ class QuantifiedExpression(VariableBinderExpression):
             term = term.term
         return (
             self.getQuantifier()
-            + ' '
-            + ' '.join("%s" % v for v in variables)
+            + " "
+            + " ".join("%s" % v for v in variables)
             + Tokens.DOT
             + "%s" % term
         )
@@ -1769,7 +1755,6 @@ class AllExpression(QuantifiedExpression):
         return Tokens.ALL
 
 
-@python_2_unicode_compatible
 class NegatedExpression(Expression):
     def __init__(self, term):
         assert isinstance(term, Expression), "%s is not an Expression" % term
@@ -1814,7 +1799,6 @@ class NegatedExpression(Expression):
         return Tokens.NOT + "%s" % self.term
 
 
-@python_2_unicode_compatible
 class BinaryExpression(Expression):
     def __init__(self, first, second):
         assert isinstance(first, Expression), "%s is not an Expression" % first
@@ -1857,7 +1841,7 @@ class BinaryExpression(Expression):
     def __str__(self):
         first = self._str_subex(self.first)
         second = self._str_subex(self.second)
-        return Tokens.OPEN + first + ' ' + self.getOp() + ' ' + second + Tokens.CLOSE
+        return Tokens.OPEN + first + " " + self.getOp() + " " + second + Tokens.CLOSE
 
     def _str_subex(self, subex):
         return "%s" % subex
@@ -1955,7 +1939,7 @@ class UnexpectedTokenException(LogicalExpressionException):
         elif unexpected:
             msg = "Unexpected token: '%s'." % unexpected
             if message:
-                msg += '  ' + message
+                msg += "  " + message
         else:
             msg = "Expected token '%s'." % expected
         LogicalExpressionException.__init__(self, index, msg)
@@ -1964,9 +1948,9 @@ class UnexpectedTokenException(LogicalExpressionException):
 class ExpectedMoreTokensException(LogicalExpressionException):
     def __init__(self, index, message=None):
         if not message:
-            message = 'More tokens expected.'
+            message = "More tokens expected."
         LogicalExpressionException.__init__(
-            self, index, 'End of input found.  ' + message
+            self, index, "End of input found.  " + message
         )
 
 
@@ -1978,8 +1962,8 @@ def is_indvar(expr):
     :param expr: str
     :return: bool True if expr is of the correct form
     """
-    assert isinstance(expr, string_types), "%s is not a string" % expr
-    return re.match(r'^[a-df-z]\d*$', expr) is not None
+    assert isinstance(expr, str), "%s is not a string" % expr
+    return re.match(r"^[a-df-z]\d*$", expr) is not None
 
 
 def is_funcvar(expr):
@@ -1990,8 +1974,8 @@ def is_funcvar(expr):
     :param expr: str
     :return: bool True if expr is of the correct form
     """
-    assert isinstance(expr, string_types), "%s is not a string" % expr
-    return re.match(r'^[A-Z]\d*$', expr) is not None
+    assert isinstance(expr, str), "%s is not a string" % expr
+    return re.match(r"^[A-Z]\d*$", expr) is not None
 
 
 def is_eventvar(expr):
@@ -2002,71 +1986,71 @@ def is_eventvar(expr):
     :param expr: str
     :return: bool True if expr is of the correct form
     """
-    assert isinstance(expr, string_types), "%s is not a string" % expr
-    return re.match(r'^e\d*$', expr) is not None
+    assert isinstance(expr, str), "%s is not a string" % expr
+    return re.match(r"^e\d*$", expr) is not None
 
 
 def demo():
     lexpr = Expression.fromstring
-    print('=' * 20 + 'Test reader' + '=' * 20)
-    print(lexpr(r'john'))
-    print(lexpr(r'man(x)'))
-    print(lexpr(r'-man(x)'))
-    print(lexpr(r'(man(x) & tall(x) & walks(x))'))
-    print(lexpr(r'exists x.(man(x) & tall(x) & walks(x))'))
-    print(lexpr(r'\x.man(x)'))
-    print(lexpr(r'\x.man(x)(john)'))
-    print(lexpr(r'\x y.sees(x,y)'))
-    print(lexpr(r'\x y.sees(x,y)(a,b)'))
-    print(lexpr(r'(\x.exists y.walks(x,y))(x)'))
-    print(lexpr(r'exists x.x = y'))
-    print(lexpr(r'exists x.(x = y)'))
-    print(lexpr('P(x) & x=y & P(y)'))
-    print(lexpr(r'\P Q.exists x.(P(x) & Q(x))'))
-    print(lexpr(r'man(x) <-> tall(x)'))
+    print("=" * 20 + "Test reader" + "=" * 20)
+    print(lexpr(r"john"))
+    print(lexpr(r"man(x)"))
+    print(lexpr(r"-man(x)"))
+    print(lexpr(r"(man(x) & tall(x) & walks(x))"))
+    print(lexpr(r"exists x.(man(x) & tall(x) & walks(x))"))
+    print(lexpr(r"\x.man(x)"))
+    print(lexpr(r"\x.man(x)(john)"))
+    print(lexpr(r"\x y.sees(x,y)"))
+    print(lexpr(r"\x y.sees(x,y)(a,b)"))
+    print(lexpr(r"(\x.exists y.walks(x,y))(x)"))
+    print(lexpr(r"exists x.x = y"))
+    print(lexpr(r"exists x.(x = y)"))
+    print(lexpr("P(x) & x=y & P(y)"))
+    print(lexpr(r"\P Q.exists x.(P(x) & Q(x))"))
+    print(lexpr(r"man(x) <-> tall(x)"))
 
-    print('=' * 20 + 'Test simplify' + '=' * 20)
-    print(lexpr(r'\x.\y.sees(x,y)(john)(mary)').simplify())
-    print(lexpr(r'\x.\y.sees(x,y)(john, mary)').simplify())
-    print(lexpr(r'all x.(man(x) & (\x.exists y.walks(x,y))(x))').simplify())
-    print(lexpr(r'(\P.\Q.exists x.(P(x) & Q(x)))(\x.dog(x))(\x.bark(x))').simplify())
+    print("=" * 20 + "Test simplify" + "=" * 20)
+    print(lexpr(r"\x.\y.sees(x,y)(john)(mary)").simplify())
+    print(lexpr(r"\x.\y.sees(x,y)(john, mary)").simplify())
+    print(lexpr(r"all x.(man(x) & (\x.exists y.walks(x,y))(x))").simplify())
+    print(lexpr(r"(\P.\Q.exists x.(P(x) & Q(x)))(\x.dog(x))(\x.bark(x))").simplify())
 
-    print('=' * 20 + 'Test alpha conversion and binder expression equality' + '=' * 20)
-    e1 = lexpr('exists x.P(x)')
+    print("=" * 20 + "Test alpha conversion and binder expression equality" + "=" * 20)
+    e1 = lexpr("exists x.P(x)")
     print(e1)
-    e2 = e1.alpha_convert(Variable('z'))
+    e2 = e1.alpha_convert(Variable("z"))
     print(e2)
     print(e1 == e2)
 
 
 def demo_errors():
-    print('=' * 20 + 'Test reader errors' + '=' * 20)
-    demoException('(P(x) & Q(x)')
-    demoException('((P(x) &) & Q(x))')
-    demoException('P(x) -> ')
-    demoException('P(x')
-    demoException('P(x,')
-    demoException('P(x,)')
-    demoException('exists')
-    demoException('exists x.')
-    demoException('\\')
-    demoException('\\ x y.')
-    demoException('P(x)Q(x)')
-    demoException('(P(x)Q(x)')
-    demoException('exists x -> y')
+    print("=" * 20 + "Test reader errors" + "=" * 20)
+    demoException("(P(x) & Q(x)")
+    demoException("((P(x) &) & Q(x))")
+    demoException("P(x) -> ")
+    demoException("P(x")
+    demoException("P(x,")
+    demoException("P(x,)")
+    demoException("exists")
+    demoException("exists x.")
+    demoException("\\")
+    demoException("\\ x y.")
+    demoException("P(x)Q(x)")
+    demoException("(P(x)Q(x)")
+    demoException("exists x -> y")
 
 
 def demoException(s):
     try:
         Expression.fromstring(s)
     except LogicalExpressionException as e:
-        print("%s: %s" % (e.__class__.__name__, e))
+        print(f"{e.__class__.__name__}: {e}")
 
 
 def printtype(ex):
-    print("%s : %s" % (ex.str(), ex.type))
+    print(f"{ex.str()} : {ex.type}")
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     demo()
 #    demo_errors()
