@@ -1,8 +1,8 @@
 # Natural Language Toolkit: Stemmers
 #
-# Copyright (C) 2001-2020 NLTK Project
+# Copyright (C) 2001-2022 NLTK Project
 # Author: Steven Tomcavage <stomcava@law.upenn.edu>
-# URL: <http://nltk.org/>
+# URL: <https://www.nltk.org/>
 # For license information, see LICENSE.TXT
 
 """
@@ -170,8 +170,7 @@ class LancasterStemmer(StemmerI):
     )
 
     def __init__(self, rule_tuple=None, strip_prefix_flag=False):
-        """Create an instance of the Lancaster stemmer.
-        """
+        """Create an instance of the Lancaster stemmer."""
         # Setup an empty rule dictionary - this will be filled in later
         self.rule_dictionary = {}
         # Check if a user wants to strip prefix
@@ -189,13 +188,13 @@ class LancasterStemmer(StemmerI):
         """
         # If there is no argument for the function, use class' own rule tuple.
         rule_tuple = rule_tuple if rule_tuple else self._rule_tuple
-        valid_rule = re.compile("^[a-z]+\*?\d[a-z]*[>\.]?$")
+        valid_rule = re.compile(r"^[a-z]+\*?\d[a-z]*[>\.]?$")
         # Empty any old rules from the rule set before adding new ones
         self.rule_dictionary = {}
 
         for rule in rule_tuple:
             if not valid_rule.match(rule):
-                raise ValueError("The rule {0} is invalid".format(rule))
+                raise ValueError(f"The rule {rule} is invalid")
             first_letter = rule[0:1]
             if first_letter in self.rule_dictionary:
                 self.rule_dictionary[first_letter].append(rule)
@@ -203,8 +202,7 @@ class LancasterStemmer(StemmerI):
                 self.rule_dictionary[first_letter] = [rule]
 
     def stem(self, word):
-        """Stem a word using the Lancaster stemmer.
-        """
+        """Stem a word using the Lancaster stemmer."""
         # Lower-case the word, since all the rules are lower-cased
         word = word.lower()
         word = self.__stripPrefix(word) if self._strip_prefix else word
@@ -219,10 +217,9 @@ class LancasterStemmer(StemmerI):
         return self.__doStemming(word, intact_word)
 
     def __doStemming(self, word, intact_word):
-        """Perform the actual word stemming
-        """
+        """Perform the actual word stemming"""
 
-        valid_rule = re.compile("^([a-z]+)(\*?)(\d)([a-z]*)([>\.]?)$")
+        valid_rule = re.compile(r"^([a-z]+)(\*?)(\d)([a-z]*)([>\.]?)$")
 
         proceed = True
 
@@ -284,8 +281,7 @@ class LancasterStemmer(StemmerI):
         return word
 
     def __getLastLetter(self, word):
-        """Get the zero-based index of the last alphabetic character in this string
-        """
+        """Get the zero-based index of the last alphabetic character in this string"""
         last_letter = -1
         for position in range(len(word)):
             if word[position].isalpha():
@@ -295,8 +291,7 @@ class LancasterStemmer(StemmerI):
         return last_letter
 
     def __isAcceptable(self, word, remove_total):
-        """Determine if the word is acceptable for stemming.
-        """
+        """Determine if the word is acceptable for stemming."""
         word_is_acceptable = False
         # If the word starts with a vowel, it must be at least 2
         # characters long to be stemmed
@@ -313,8 +308,7 @@ class LancasterStemmer(StemmerI):
         return word_is_acceptable
 
     def __applyRule(self, word, remove_total, append_string):
-        """Apply the stemming rule to the word
-        """
+        """Apply the stemming rule to the word"""
         # Remove letters from the end of the word
         new_word_length = len(word) - remove_total
         word = word[0:new_word_length]
