@@ -1,17 +1,15 @@
-# -*- coding: utf-8 -*-
-
 # Natural Language Toolkit: RSLP Stemmer
 #
-# Copyright (C) 2001-2019 NLTK Project
+# Copyright (C) 2001-2022 NLTK Project
 # Author: Tiago Tresoldi <tresoldi@gmail.com>
-# URL: <http://nltk.org/>
+# URL: <https://www.nltk.org/>
 # For license information, see LICENSE.TXT
 
 # This code is based on the algorithm presented in the paper "A Stemming
 # Algorithm for the Portuguese Language" by Viviane Moreira Orengo and
 # Christian Huyck, which unfortunately I had no access to. The code is a
 # Python version, with some minor modifications of mine, to the description
-# presented at http://www.webcitation.org/5NnvdIzOb and to the C source code
+# presented at https://www.webcitation.org/5NnvdIzOb and to the C source code
 # available at http://www.inf.ufrgs.br/~arcoelho/rslp/integrando_rslp.html.
 # Please note that this stemmer is intended for demonstration and educational
 # purposes only. Feel free to write me for any comments, including the
@@ -22,7 +20,7 @@
 # Algorithm for the Portuguese Language" de Viviane Moreira Orengo e
 # Christian Huyck, o qual infelizmente não tive a oportunidade de ler. O
 # código é uma conversão para Python, com algumas pequenas modificações
-# minhas, daquele apresentado em http://www.webcitation.org/5NnvdIzOb e do
+# minhas, daquele apresentado em https://www.webcitation.org/5NnvdIzOb e do
 # código para linguagem C disponível em
 # http://www.inf.ufrgs.br/~arcoelho/rslp/integrando_rslp.html. Por favor,
 # lembre-se de que este stemmer foi desenvolvido com finalidades unicamente
@@ -30,9 +28,8 @@
 # comentário, inclusive sobre o desenvolvimento de um stemmer diferente
 # e/ou melhor para o português. Também sugiro utilizar-se a lista de discussão
 # do NLTK para o português para qualquer debate.
-from __future__ import print_function, unicode_literals
-from nltk.data import load
 
+from nltk.data import load
 from nltk.stem.api import StemmerI
 
 
@@ -65,7 +62,7 @@ class RSLPStemmer(StemmerI):
         self._model.append(self.read_rule("step6.pt"))
 
     def read_rule(self, filename):
-        rules = load('nltk:stemmers/rslp/' + filename, format='raw').decode("utf8")
+        rules = load("nltk:stemmers/rslp/" + filename, format="raw").decode("utf8")
         lines = rules.split("\n")
 
         lines = [line for line in lines if line != ""]  # remove blank lines
