@@ -1,11 +1,10 @@
-# -*- coding: utf-8 -*-
 # Natural Language Toolkit: Interface to the HunPos POS-tagger
 #
-# Copyright (C) 2001-2020 NLTK Project
+# Copyright (C) 2001-2022 NLTK Project
 # Author: Peter Ljunglöf <peter.ljunglof@heatherleaf.se>
 #         Dávid Márk Nemeskey <nemeskeyd@gmail.com> (modifications)
 #         Attila Zséder <zseder@gmail.com> (modifications)
-# URL: <http://nltk.org/>
+# URL: <https://www.nltk.org/>
 # For license information, see LICENSE.TXT
 
 """
@@ -13,12 +12,12 @@ A module for interfacing with the HunPos open-source POS-tagger.
 """
 
 import os
-from subprocess import Popen, PIPE
+from subprocess import PIPE, Popen
 
 from nltk.internals import find_binary, find_file
 from nltk.tag.api import TaggerI
 
-_hunpos_url = "http://code.google.com/p/hunpos/"
+_hunpos_url = "https://code.google.com/p/hunpos/"
 
 _hunpos_charset = "ISO-8859-1"
 """The default encoding used by hunpos: ISO-8859-1."""
@@ -137,13 +136,3 @@ class HunposTagger(TaggerI):
         self._hunpos.stdout.readline()
 
         return tagged_tokens
-
-
-# skip doctests if Hunpos tagger is not installed
-def setup_module(module):
-    from nose import SkipTest
-
-    try:
-        HunposTagger("en_wsj.model")
-    except LookupError:
-        raise SkipTest("HunposTagger is not available")
