@@ -1,9 +1,9 @@
 # Natural Language Toolkit: Simple Tokenizers
 #
-# Copyright (C) 2001-2020 NLTK Project
+# Copyright (C) 2001-2022 NLTK Project
 # Author: Edward Loper <edloper@gmail.com>
 #         Steven Bird <stevenbird1@gmail.com>
-# URL: <http://nltk.sourceforge.net>
+# URL: <https://www.nltk.org>
 # For license information, see LICENSE.TXT
 
 r"""
@@ -35,8 +35,8 @@ to specify the tokenization conventions when building a `CorpusReader`.
 
 """
 
-from nltk.tokenize.api import TokenizerI, StringTokenizer
-from nltk.tokenize.util import string_span_tokenize, regexp_span_tokenize
+from nltk.tokenize.api import StringTokenizer, TokenizerI
+from nltk.tokenize.util import regexp_span_tokenize, string_span_tokenize
 
 
 class SpaceTokenizer(StringTokenizer):
@@ -74,8 +74,7 @@ class CharTokenizer(StringTokenizer):
         return list(s)
 
     def span_tokenize(self, s):
-        for i, j in enumerate(range(1, len(s) + 1)):
-            yield i, j
+        yield from enumerate(range(1, len(s) + 1))
 
 
 class LineTokenizer(TokenizerI):
@@ -123,11 +122,9 @@ class LineTokenizer(TokenizerI):
     # discard-eof not implemented
     def span_tokenize(self, s):
         if self._blanklines == "keep":
-            for span in string_span_tokenize(s, r"\n"):
-                yield span
+            yield from string_span_tokenize(s, r"\n")
         else:
-            for span in regexp_span_tokenize(s, r"\n(\s+\n)*"):
-                yield span
+            yield from regexp_span_tokenize(s, r"\n(\s+\n)*")
 
 
 ######################################################################
