@@ -1,33 +1,31 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 # Natural Language Toolkit: Interface to the Stanford Segmenter
 # for Chinese and Arabic
 #
-# Copyright (C) 2001-2020 NLTK Project
+# Copyright (C) 2001-2022 NLTK Project
 # Author: 52nlp <52nlpcn@gmail.com>
 #         Casper Lehmann-Strøm <casperlehmann@gmail.com>
 #         Alex Constantin <alex@keyworder.ch>
 #
-# URL: <http://nltk.org/>
+# URL: <https://www.nltk.org/>
 # For license information, see LICENSE.TXT
 
-import tempfile
-import os
 import json
+import os
+import tempfile
 import warnings
 from subprocess import PIPE
 
 from nltk.internals import (
-    find_jar,
-    find_file,
-    find_dir,
-    config_java,
-    java,
     _java_options,
+    config_java,
+    find_dir,
+    find_file,
+    find_jar,
+    java,
 )
 from nltk.tokenize.api import TokenizerI
 
-
 _stanford_url = "https://nlp.stanford.edu/software"
 
 
@@ -120,12 +118,12 @@ class StanfordSegmenter(TokenizerI):
         self.java_options = java_options
         options = {} if options is None else options
         self._options_cmd = ",".join(
-            "{0}={1}".format(key, json.dumps(val)) for key, val in options.items()
+            f"{key}={json.dumps(val)}" for key, val in options.items()
         )
 
     def default_config(self, lang):
         """
-        Attempt to intialize Stanford Word Segmenter for the specified language
+        Attempt to initialize Stanford Word Segmenter for the specified language
         using the STANFORD_SEGMENTER and STANFORD_MODELS environment variables
         """
 
@@ -158,12 +156,12 @@ class StanfordSegmenter(TokenizerI):
                     verbose=False,
                     env_vars=("STANFORD_MODELS",),
                 )
-            except LookupError:
+            except LookupError as e:
                 raise LookupError(
                     "Could not find '%s' (tried using env. "
                     "variables STANFORD_MODELS and <STANFORD_SEGMENTER>/data/)"
                     % path_to_dict
-                )
+                ) from e
 
             sihan_dir = "./data/"
             try:
@@ -174,13 +172,13 @@ class StanfordSegmenter(TokenizerI):
                     env_vars=("STANFORD_SEGMENTER",),
                 )
                 self._sihan_corpora_dict = os.path.join(path_to_sihan_dir, sihan_dir)
-            except LookupError:
+            except LookupError as e:
                 raise LookupError(
                     "Could not find '%s' (tried using the "
                     "STANFORD_SEGMENTER environment variable)" % sihan_dir
-                )
+                ) from e
         else:
-            raise LookupError("Unsupported language {}".format(lang))
+            raise LookupError(f"Unsupported language {lang}")
 
         try:
             self._model = find_file(
@@ -190,18 +188,17 @@ class StanfordSegmenter(TokenizerI):
                 verbose=False,
                 env_vars=("STANFORD_MODELS", "STANFORD_SEGMENTER"),
             )
-        except LookupError:
+        except LookupError as e:
             raise LookupError(
                 "Could not find '%s' (tried using env. "
                 "variables STANFORD_MODELS and <STANFORD_SEGMENTER>/data/)" % model
-            )
+            ) from e
 
     def tokenize(self, s):
         super().tokenize(s)
 
     def segment_file(self, input_file_path):
-        """
-        """
+        """ """
         cmd = [
             self._java_class,
             "-loadClassifier",
@@ -231,15 +228,14 @@ class StanfordSegmenter(TokenizerI):
         return self.segment_sents([tokens])
 
     def segment_sents(self, sentences):
-        """
-        """
+        """ """
         encoding = self._encoding
         # Create a temporary input file
         _input_fh, self._input_file_path = tempfile.mkstemp(text=True)
 
         # Write the actural sentences to the temporary input file
         _input_fh = os.fdopen(_input_fh, "wb")
-        _input = "\n".join((" ".join(x) for x in sentences))
+        _input = "\n".join(" ".join(x) for x in sentences)
         if isinstance(_input, str) and encoding:
             _input = _input.encode(encoding)
         _input_fh.write(_input)
@@ -294,16 +290,3 @@ class StanfordSegmenter(TokenizerI):
         config_java(options=default_options, verbose=False)
 
         return stdout
-
-
-def setup_module(module):
-    from nose import SkipTest
-
-    try:
-        seg = StanfordSegmenter()
-        seg.default_config("ar")
-        seg.default_config("zh")
-    except LookupError as e:
-        raise SkipTest(
-            "Tests for nltk.tokenize.stanford_segmenter skipped: %s" % str(e)
-        )
