@@ -1,9 +1,8 @@
-# -*- coding: utf-8 -*-
 # Natural Language Toolkit: Tokenizer Utilities
 #
-# Copyright (C) 2001-2019 NLTK Project
+# Copyright (C) 2001-2022 NLTK Project
 # Author: Steven Bird <stevenbird1@gmail.com>
-# URL: <http://nltk.sourceforge.net>
+# URL: <https://www.nltk.org>
 # For license information, see LICENSE.TXT
 
 from re import finditer
@@ -93,10 +92,10 @@ def spans_to_relative(spans):
         prev = right
 
 
-class CJKChars(object):
+class CJKChars:
     """
     An object that enumerates the code points of the CJK characters as listed on
-    http://en.wikipedia.org/wiki/Basic_Multilingual_Plane#Basic_Multilingual_Plane
+    https://en.wikipedia.org/wiki/Basic_Multilingual_Plane#Basic_Multilingual_Plane
 
     This is a Python port of the CJK code point enumerations of Moses tokenizer:
     https://github.com/moses-smt/mosesdecoder/blob/master/scripts/tokenizer/detokenizer.perl#L309
@@ -289,8 +288,8 @@ def align_tokens(tokens, sentence):
     for token in tokens:
         try:
             start = sentence.index(token, point)
-        except ValueError:
-            raise ValueError('substring "{}" not found in "{}"'.format(token, sentence))
+        except ValueError as e:
+            raise ValueError(f'substring "{token}" not found in "{sentence}"') from e
         point = start + len(token)
         offsets.append((start, point))
     return offsets
