@@ -2,9 +2,9 @@
 #
 # Natural Language Toolkit: Deprecated Function & Class Finder
 #
-# Copyright (C) 2001-2020 NLTK Project
+# Copyright (C) 2001-2022 NLTK Project
 # Author: Edward Loper <edloper@gmail.com>
-# URL: <http://nltk.org/>
+# URL: <https://www.nltk.org/>
 # For license information, see LICENSE.TXT
 
 
@@ -25,11 +25,13 @@ identifier will be highlighted in red.
 import os
 import re
 import sys
-import tokenize
 import textwrap
-import nltk.corpus
+import tokenize
 from doctest import DocTestParser, register_optionflag
+
 from cStringIO import StringIO
+
+import nltk.corpus
 from nltk import defaultdict
 
 ######################################################################
@@ -38,32 +40,32 @@ from nltk import defaultdict
 
 #: A little over-simplified, but it'll do.
 STRING_PAT = (
-    r'\s*[ur]{0,2}(?:'
-    '"""[\s\S]*?"""|'
+    r"\s*[ur]{0,2}(?:"
+    r'"""[\s\S]*?"""|'
     '"[^"\n]+?"|'
-    "'''[\s\S]*?'''|"
+    r"'''[\s\S]*?'''|"
     "'[^'\n]+?'"
-    ")\s*"
+    r")\s*"
 )
 STRING_RE = re.compile(STRING_PAT)
 
-STRINGS_PAT = '{}(?:[+]?{})*'.format(STRING_PAT, STRING_PAT)
+STRINGS_PAT = f"{STRING_PAT}(?:[+]?{STRING_PAT})*"
 STRINGS_RE = re.compile(STRINGS_PAT)
 
 # Define a regexp to search for deprecated definitions.
 DEPRECATED_DEF_PAT = (
-    r'^\s*@deprecated\s*\(\s*({})\s*\)\s*\n+'.format(STRINGS_PAT)
-    + r'\s*def\s*(\w+).*'
-    + r'|'
-    + r'^\s*class\s+(\w+)\s*\(.*Deprecated.*\):\s*'
+    fr"^\s*@deprecated\s*\(\s*({STRINGS_PAT})\s*\)\s*\n+"
+    + r"\s*def\s*(\w+).*"
+    + r"|"
+    + r"^\s*class\s+(\w+)\s*\(.*Deprecated.*\):\s*"
 )
 DEPRECATED_DEF_RE = re.compile(DEPRECATED_DEF_PAT, re.MULTILINE)
 
 CORPUS_READ_METHOD_RE = re.compile(
-    '({})\.read\('.format('|'.join(re.escape(n) for n in dir(nltk.corpus)))
+    r"({})\.read\(".format("|".join(re.escape(n) for n in dir(nltk.corpus)))
 )
 
-CLASS_DEF_RE = re.compile('^\s*class\s+(\w+)\s*[:\(]')
+CLASS_DEF_RE = re.compile(r"^\s*class\s+(\w+)\s*[:\(]")
 
 ######################################################################
 # Globals
@@ -81,7 +83,7 @@ except ImportError:
 
     class TerminalController:
         def __getattr__(self, attr):
-            return ''
+            return ""
 
 
 term = TerminalController()
@@ -90,10 +92,6 @@ term = TerminalController()
 # Code
 ######################################################################
 
-# If we're using py24, then ignore the +SKIP directive.
-if sys.version_info[:2] < (2, 5):
-    register_optionflag('SKIP')
-
 
 def strip_quotes(s):
     s = s.strip()
@@ -106,13 +104,13 @@ def strip_quotes(s):
 
 
 def find_class(s, index):
-    lines = s[:index].split('\n')
+    lines = s[:index].split("\n")
     while lines:
         m = CLASS_DEF_RE.match(lines[-1])
         if m:
-            return m.group(1) + '.'
+            return m.group(1) + "."
         lines.pop()
-    return '?.'
+    return "?."
 
 
 def find_deprecated_defs(pkg_dir):
@@ -124,30 +122,30 @@ def find_deprecated_defs(pkg_dir):
     # Walk through the directory, finding python files.
     for root, dirs, files in os.walk(pkg_dir):
         for filename in files:
-            if filename.endswith('.py'):
+            if filename.endswith(".py"):
                 # Search the file for any deprecated definitions.
                 s = open(os.path.join(root, filename)).read()
                 for m in DEPRECATED_DEF_RE.finditer(s):
                     if m.group(2):
                         name = m.group(2)
-                        msg = ' '.join(
+                        msg = " ".join(
                             strip_quotes(s) for s in STRING_RE.findall(m.group(1))
                         )
-                        msg = ' '.join(msg.split())
-                        if m.group()[0] in ' \t':
+                        msg = " ".join(msg.split())
+                        if m.group()[0] in " \t":
                             cls = find_class(s, m.start())
-                            deprecated_methods[name].add((msg, cls, '()'))
+                            deprecated_methods[name].add((msg, cls, "()"))
                         else:
-                            deprecated_funcs[name].add((msg, '', '()'))
+                            deprecated_funcs[name].add((msg, "", "()"))
                     else:
                         name = m.group(3)
                         m2 = STRING_RE.match(s, m.end())
                         if m2:
                             msg = strip_quotes(m2.group())
                         else:
-                            msg = ''
-                        msg = ' '.join(msg.split())
-                        deprecated_classes[name].add((msg, '', ''))
+                            msg = ""
+                        msg = " ".join(msg.split())
+                        deprecated_classes[name].add((msg, "", ""))
 
 
 def print_deprecated_uses(paths):
@@ -158,9 +156,9 @@ def print_deprecated_uses(paths):
             dep_names.update(
                 print_deprecated_uses([os.path.join(path, f) for f in os.listdir(path)])
             )
-        elif path.endswith('.py'):
+        elif path.endswith(".py"):
             print_deprecated_uses_in(open(path).readline, path, dep_files, dep_names, 0)
-        elif path.endswith('.doctest') or path.endswith('.txt'):
+        elif path.endswith(".doctest") or path.endswith(".txt"):
             for example in DocTestParser().get_examples(open(path).read()):
                 ex = StringIO(example.source)
                 try:
@@ -169,15 +167,15 @@ def print_deprecated_uses(paths):
                     )
                 except tokenize.TokenError:
                     print(
-                        term.RED + 'Caught TokenError -- '
-                        'malformatted doctest?' + term.NORMAL
+                        term.RED + "Caught TokenError -- "
+                        "malformatted doctest?" + term.NORMAL
                     )
     return dep_names
 
 
 def print_deprecated_uses_in(readline, path, dep_files, dep_names, lineno_offset):
     tokiter = tokenize.generate_tokens(readline)
-    context = ['']
+    context = [""]
     for (typ, tok, start, end, line) in tokiter:
         # Remember the previous line -- it might contain
         # the @deprecated decorator.
@@ -189,25 +187,23 @@ def print_deprecated_uses_in(readline, path, dep_files, dep_names, lineno_offset
         # Ignore all tokens except deprecated names.
         if not (
             tok in deprecated_classes
-            or (
-                tok in deprecated_funcs and re.search(r'\b{}\s*\('.format(esctok), line)
-            )
+            or (tok in deprecated_funcs and re.search(fr"\b{esctok}\s*\(", line))
             or (
                 tok in deprecated_methods
-                and re.search(r'(?!<\bself)[.]\s*{}\s*\('.format(esctok), line)
+                and re.search(fr"(?!<\bself)[.]\s*{esctok}\s*\(", line)
             )
         ):
             continue
         # Hack: only complain about read if it's used after a corpus.
-        if tok == 'read' and not CORPUS_READ_METHOD_RE.search(line):
+        if tok == "read" and not CORPUS_READ_METHOD_RE.search(line):
             continue
         # Ignore deprecated definitions:
-        if DEPRECATED_DEF_RE.search(''.join(context)):
+        if DEPRECATED_DEF_RE.search("".join(context)):
             continue
         # Print a header for the first use in a file:
         if path not in dep_files:
-            print('\n' + term.BOLD + path + term.NORMAL)
-            print('  {}linenum{}'.format(term.YELLOW, term.NORMAL))
+            print("\n" + term.BOLD + path + term.NORMAL)
+            print(f"  {term.YELLOW}linenum{term.NORMAL}")
             dep_files.add(path)
         # Mark the offending token.
         dep_names.add(tok)
@@ -216,34 +212,34 @@ def print_deprecated_uses_in(readline, path, dep_files, dep_names, lineno_offset
         elif term.BOLD:
             sub = term.BOLD + tok + term.NORMAL
         else:
-            sub = '<<' + tok + '>>'
-        line = re.sub(r'\b{}\b'.format(esctok), sub, line)
+            sub = "<<" + tok + ">>"
+        line = re.sub(fr"\b{esctok}\b", sub, line)
         # Print the offending line.
         print(
-            '  {}[{:5d}]{} {}'.format(
+            "  {}[{:5d}]{} {}".format(
                 term.YELLOW, start[0] + lineno_offset, term.NORMAL, line.rstrip()
             )
         )
 
 
 def main():
-    paths = sys.argv[1:] or ['.']
+    paths = sys.argv[1:] or ["."]
 
-    print('Importing nltk...')
+    print("Importing nltk...")
     try:
         import nltk
     except ImportError:
-        print('Unable to import nltk -- check your PYTHONPATH.')
+        print("Unable to import nltk -- check your PYTHONPATH.")
         sys.exit(-1)
 
-    print('Finding definitions of deprecated funtions & classes in nltk...')
+    print("Finding definitions of deprecated functions & classes in nltk...")
     find_deprecated_defs(nltk.__path__[0])
 
-    print('Looking for possible uses of deprecated funcs & classes...')
+    print("Looking for possible uses of deprecated funcs & classes...")
     dep_names = print_deprecated_uses(paths)
 
     if not dep_names:
-        print('No deprecated funcs or classes found!')
+        print("No deprecated funcs or classes found!")
     else:
         print("\n" + term.BOLD + "What you should use instead:" + term.NORMAL)
         for name in sorted(dep_names):
@@ -255,13 +251,13 @@ def main():
             for msg, prefix, suffix in msgs:
                 print(
                     textwrap.fill(
-                        term.RED + prefix + name + suffix + term.NORMAL + ': ' + msg,
+                        term.RED + prefix + name + suffix + term.NORMAL + ": " + msg,
                         width=75,
-                        initial_indent=' ' * 2,
-                        subsequent_indent=' ' * 6,
+                        initial_indent=" " * 2,
+                        subsequent_indent=" " * 6,
                     )
                 )
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main()
